//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_ERROR_HANDLER_H
#define LIBUNCIA_ERROR_HANDLER_H

#include <libuncia/quit.h>

namespace libuncia
{
    /**
      * The error_handler_set function is used to set the object to be
      * used to handle errors found by the library.
      *
      * @param newhandler
      *     A pointer to an object in the quit hierarchy.  This object
      *     will be called when an error occurs anywhere within the
      *     libuncia library.
      *
      *     The lifetime of this object must exceed the lifetime of
      *     any objects operton by the libuncia library, or until
      *     error_handler_set is called again to redirect errors to
      *     another object.
      *
      *     The NULL pointer may be used to request that error handling
      *     once again be handled by the libuncia library's default
      *     error handler, which prints error messages to stderr and
      *     terminates via the ::exit(2) system call.
      */
    void error_handler_set(quit *newhandler);

    /**
      * The error_handler_get function is used to obtain a pointer to the
      * current error handler.
      *
      * This may be used to temporarily redirect errors, and then
      * restore the error handler.
      */
    quit *error_handler_get();
}

#endif // LIBUNCIA_ERROR_HANDLER_H
