//
//      uncia - a big cat
//      Copyright (C) 2008 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#include <string>

#include <libuncia/input/catenate.h>


libuncia::input_catenate::~input_catenate()
{
}


libuncia::input_catenate::input_catenate(const pointer &d1, const pointer &d2) :
    deeper1(d1),
    deeper2(d2)
{
}


libuncia::input_catenate::pointer
libuncia::input_catenate::create(const pointer &d1, const pointer &d2)
{
    return pointer(new input_catenate(d1, d2));
}


ssize_t
libuncia::input_catenate::read(void *data, size_t size)
{
    if (deeper1)
    {
        ssize_t e1 = deeper1->read(data, size);
        if (e1 != 0)
            return e1;
        deeper1.reset();
    }

    //
    // We keep deeper2 open even after EOF so that we can still report
    // errors against a named file.
    //
    return deeper2->read(data, size);
}


std::string
libuncia::input_catenate::filename_get()
    const
{
    return (deeper1 ? deeper1 : deeper2)->filename_get();
}
