//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_INPUT_CATENATE_H
#define LIBUNCIA_INPUT_CATENATE_H

#include <libuncia/input.h>

namespace libuncia
{
    /**
      * The input_catenate class is used to represent an input
      * constructed of two deeper inputs, concatenated head-to-tail.
      */
    class input_catenate:
        public input
    {
    public:
        /**
          * The destructor.
          */
        virtual ~input_catenate();

    private:
        /**
          * The constructor.
          *
          * @param d1
          *     The first input to be read.
          * @param d2
          *     The second input to be read.
          */
        input_catenate(const pointer &d1, const pointer &d2);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param d1
          *     The first input to be read.
          * @param d2
          *     The second input to be read.
          */
        static pointer create(const pointer &d1, const pointer &d2);

    protected:
        // See base class for documentation.
        ssize_t read(void *data, size_t size);

        // See base class for documentation.
        std::string filename_get() const;

    private:
        /**
          * The deeper1 instance variable is used to remember the first
          * input source to be consumed.
          */
        pointer deeper1;

        /**
          * The deeper2 instance variable is used to remember the second
          * input source to be consumed.
          */
        pointer deeper2;

        /**
          * The default constructor.  Do not use.
          */
        input_catenate();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        input_catenate(const input_catenate &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        input_catenate &operator=(const input_catenate &rhs);
    };
}

#endif // LIBUNCIA_INPUT_CATENATE_H
