//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_INPUT_ECHO_H
#define LIBUNCIA_INPUT_ECHO_H

#include <string>
#include <libuncia/input.h>

namespace libuncia
{
    /**
      * The input_echo class is used to represent an input sourced from
      * a string.
      */
    class input_echo:
        public input
    {
    public:
        /**
          * The destructor.
          */
        virtual ~input_echo();

    private:
        /**
          * The constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param text
          *     The data to be used as input.
          */
        input_echo(const std::string &text);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param text
          *     The data to be used as input.
          */
        static pointer create(const std::string &text);

    protected:
        // See base class for documentation.
        ssize_t read(void *data, size_t size);

        // See base class for documentation.
        std::string filename_get() const;

    private:
        /**
          * The text instance variable is used to remember the data to
          * be used as input.
          */
        std::string text;

        /**
          * The position instance variable is used to remember where we
          * are up to in the #text to be input.
          */
        size_t position;

        /**
          * The default constructor.  Do not use.
          */
        input_echo();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        input_echo(const input_echo &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        input_echo &operator=(const input_echo &rhs);
    };
}

#endif // LIBUNCIA_INPUT_ECHO_H
