//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <fcntl.h>

#include <libuncia/error_handler.h>
#include <libuncia/input/curl.h>
#include <libuncia/input/file.h>
#include <libuncia/input/null.h>
#include <libuncia/input/stdin.h>


libuncia::input_file::~input_file()
{
    if (close(fd) < 0)
        error_handler_get()->fatal_error_errno("close %s", filename.c_str());
}


libuncia::input_file::input_file(const std::string &fn) :
    filename(fn),
    fd(-1)
{
    fd = open(filename.c_str(), O_RDONLY);
    if (fd < 0)
        error_handler_get()->fatal_error_errno("open %s", filename.c_str());
}


libuncia::input::pointer
libuncia::input_file::create(const std::string &fn)
{
    if (libuncia::input_curl::candidate(fn))
        return libuncia::input_curl::create(fn);
    if
    (
        fn == "-"
    ||
        fn == "/dev/stdin"
    ||
        fn == "/dev/fd/0"
    ||
        fn == "/proc/self/fd/0"
    )
        return libuncia::input_stdin::create();
    if (fn == "/dev/null")
        return libuncia::input_null::create();
    return pointer(new input_file(fn));
}


ssize_t
libuncia::input_file::read(void *data, size_t size)
{
    ssize_t ret = ::read(fd, data, size);
    if (ret < 0)
        error_handler_get()->fatal_error_errno("read %s", filename.c_str());
    return ret;
}


std::string
libuncia::input_file::filename_get()
    const
{
    return filename;
}
