//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <libuncia/input/filter.h>


libuncia::input_filter::~input_filter()
{
}


libuncia::input_filter::input_filter(const pointer &a_deeper) :
    deeper(a_deeper)
{
}


std::string
libuncia::input_filter::filename_get()
    const
{
    return deeper.filename_get();
}


ssize_t
libuncia::input_filter::deeper_read(void *data, size_t size)
{
    char *start = (char *)data;
    char *end = start + size;
    char *cp = start;
    while (cp < end)
    {
        if (!holding_pen.empty())
        {
            *cp++ = holding_pen.back();
            holding_pen.pop_back();
            continue;
        }

        //
        // Grab a character.  This is necessary to prime the buffer for
        // the deeper.readsome call.
        //
        if (!deeper.get(*cp))
            break;
        ++cp;

        // slurp the rest of the deeper buffer, if we can
        size_t maxlen = end - cp;
        if (maxlen)
        {
            ssize_t n = deeper.readsome(cp, maxlen);
            if (n > 0)
            {
                cp += n;
                break;
            }
        }
    }
    return (cp - start);
}
