//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <libuncia/input/filter/base64.h>


libuncia::input_filter_base64::~input_filter_base64()
{
}


libuncia::input_filter_base64::input_filter_base64(const pointer &a_deeper) :
    input_filter(a_deeper),
    residual_bits(0),
    residual_value(0),
    eof(false)
{
}


libuncia::input::pointer
libuncia::input_filter_base64::create(const pointer &a_deeper)
{
    return pointer(new input_filter_base64(a_deeper));
}


ssize_t
libuncia::input_filter_base64::read(void *data, size_t size)
{
    if (eof)
        return 0;
    unsigned char *start = (unsigned char *)data;
    unsigned char *end = start + size;
    unsigned char *cp = start;
    while (cp < end)
    {
        while (residual_bits < 8)
        {
            char c = 0;
            if (!deeper_get(c))
            {
                if (residual_bits != 0)
                    fatal_error("base64: residual bits != 0");
                eof = true;
                goto done;
            }
            switch (c)
            {
            case ' ':
            case '\t':
            case '\r':
            case '\n':
                // The RFC says to ignore white space
                continue;

            case '=':
                eof = true;
                goto done;

            case 'A':
            case 'B':
            case 'C':
            case 'D':
            case 'E':
            case 'F':
            case 'G':
            case 'H':
            case 'I':
            case 'J':
            case 'K':
            case 'L':
            case 'M':
            case 'N':
            case 'O':
            case 'P':
            case 'Q':
            case 'R':
            case 'S':
            case 'T':
            case 'U':
            case 'V':
            case 'W':
            case 'X':
            case 'Y':
            case 'Z':
                //
                // This next statement is not portable to
                // non-ascii character sets, because A-Z
                // are not guaranteed to be continuous.
                //
                c = c - 'A';
                break;

            case 'a':
            case 'b':
            case 'c':
            case 'd':
            case 'e':
            case 'f':
            case 'g':
            case 'h':
            case 'i':
            case 'j':
            case 'k':
            case 'l':
            case 'm':
            case 'n':
            case 'o':
            case 'p':
            case 'q':
            case 'r':
            case 's':
            case 't':
            case 'u':
            case 'v':
            case 'w':
            case 'x':
            case 'y':
            case 'z':
                //
                // This next statement is not portable to
                // non-ascii character sets, because a-z
                // are not guaranteed to be continuous.
                //
                c = c - 'a' + 26;
                break;

            case '0':
            case '1':
            case '2':
            case '3':
            case '4':
            case '5':
            case '6':
            case '7':
            case '8':
            case '9':
                c = c - '0' + 52;
                break;

            case '+':
                c = 62;
                break;

            case '/':
                c = 63;
                break;

            default:
                //
                // The RFC says we have the option of ignoring bogus
                // characters or whinging about them.
                //
                if (!isprint((unsigned char)c))
                    fatal_error("base64: not ASCII text");
                continue;
            }
            residual_value = (residual_value << 6) + c;
            residual_bits += 6;
        }
        residual_bits -= 8;
        *cp++ = (residual_value >> residual_bits);
    }
    done:
    return (cp - start);
}
