//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_INPUT_BASE64_H
#define LIBUNCIA_INPUT_BASE64_H

#include <libuncia/input/filter.h>

namespace libuncia
{
    /**
      * The input_filter_base64 class is used to represent and input
      * filter which decodes MIME base 64 format.
      */
    class input_filter_base64:
        public input_filter
    {
    public:
        /**
          * The destructor.
          */
        virtual ~input_filter_base64();

    private:
        /**
          * The constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param deeper
          *     where to get the input to be filtered.
          */
        input_filter_base64(const pointer &deeper);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper
          *     where to get the input to be filtered.
          */
        static pointer create(const pointer &deeper);

    protected:
        // See base class for documentation.
        ssize_t read(void *data, size_t size);

    private:
        /**
          * The residual_bits instance variable is used to remember the
          * number of bits held by the residual_value instance variable.
          */
        int residual_bits;

        /**
          * The residual_value instance variable is used to remember the
          * as-yet-unwritten bits of the value.
          */
        int residual_value;

        /**
          * The eof instance variable is used to remember whether or not
          * the end parker (=) has been seen.
          */
        bool eof;

        /**
          * The default constructor.  Do not use.
          */
        input_filter_base64();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        input_filter_base64(const input_filter_base64 &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        input_filter_base64 &operator=(const input_filter_base64 &rhs);
    };
}

#endif // LIBUNCIA_INPUT_BASE64_H
