//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <climits>

#include <libuncia/input/filter/column_remove.h>


libuncia::input_filter_column_remove::~input_filter_column_remove()
{
}


libuncia::input_filter_column_remove::input_filter_column_remove(
        const pointer &a_deeper, int a_left, int a_right) :
    input_filter(a_deeper),
    left(a_left < 0 ? 0 : a_left),
    right(a_right <= int(left) ? UINT_MAX : a_right),
    tab_width(8),
    column(0)
{
}


libuncia::input::pointer
libuncia::input_filter_column_remove::create(const pointer &a_deeper,
    int a_left, int a_right)
{
    return pointer(new input_filter_column_remove(a_deeper, a_left, a_right));
}


ssize_t
libuncia::input_filter_column_remove::read(void *data, size_t size)
{
    char *start = (char *)data;
    char *end = start + size;
    char *cp = start;
    while (cp < end)
    {
        char c = 0;
        if (!deeper_get(c))
            break;
        switch (c)
        {
        case '\t':
            {
                // fake a bunch of spaces
                unsigned newcol = ((column / tab_width) + 1) * tab_width;
                while (newcol > column)
                {
                    deeper_putback(' ');
                    --newcol;
                }
            }
            break;

        case '\r':
        case '\n':
            *cp++ = c;
            column = 0;
            break;

        case '\b':
            if (column <= left || column > right)
            {
                //
                // Note: the above comparison is intentionally
                // off-by-one to cope with boundary conditions around
                // underlined characters.
                //
                *cp++ = c;
            }
            if (column > 0)
                --column;
            break;

        default:
            if (column < left || column >= right)
                *cp++ = c;
            ++column;
            break;
        }
    }
    return (cp - start);
}
