//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <libuncia/input/filter/expand.h>


libuncia::input_filter_expand::~input_filter_expand()
{
}


libuncia::input_filter_expand::input_filter_expand(const pointer &a_deeper,
        int a_tab_width) :
    input_filter(a_deeper),
    tab_width(a_tab_width >= 1 ? a_tab_width : 8),
    column(0),
    insert_spaces(0)
{
}


libuncia::input::pointer
libuncia::input_filter_expand::create(const pointer &a_deeper, int a_tab_width)
{
    return pointer(new input_filter_expand(a_deeper, a_tab_width));
}


ssize_t
libuncia::input_filter_expand::read(void *data, size_t size)
{
    char *start = (char *)data;
    char *end = start + size;
    char *cp = start;
    while (cp < end)
    {
        if (insert_spaces > 0)
        {
            insert_one_space:
            --insert_spaces;
            *cp++ = ' ';
            ++column;
            continue;
        }

        char c = 0;
        if (!deeper_get(c))
            break;
        switch (c)
        {
        case '\t':
            {
                int new_column = ((column / tab_width) + 1) * tab_width;
                insert_spaces = new_column - column;
                goto insert_one_space;
            }

        case '\b':
            if (column > 0)
                --column;
            break;

        case '\r':
        case '\n':
            column = 0;
            break;

        default:
            ++column;
            break;
        }
        *cp++ = c;
    }
    return (cp - start);
}
