//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_INPUT_EXPAND_H
#define LIBUNCIA_INPUT_EXPAND_H

#include <libuncia/input/filter.h>

namespace libuncia
{
    /**
      * The input_filter_expand class is used to represent an input
      * filter which expands horizontal tab characters into the
      * appropriate numbers of space characters.
      */
    class input_filter_expand:
        public input_filter
    {
    public:
        /**
          * The destructor.
          */
        virtual ~input_filter_expand();

    private:
        /**
          * The constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param deeper
          *     where to obtain the input to this filter.
          * @param tab_width
          *     the width of each tab stop.
          */
        input_filter_expand(const pointer &deeper, int tab_width = 8);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper
          *     where to obtain the input to this filter.
          * @param tab_width
          *     the width of each tab stop.
          */
        static pointer create(const pointer &deeper, int tab_width = 8);

    protected:
        // See base class for documentation.
        ssize_t read(void *data, size_t size);

    private:
        /**
          * The deeper instance variable is used to remember the width
          * of each tab stop, in characters.
          */
        int tab_width;

        /**
          * The column instance variable is used to remember the current
          * input column position.
          */
        int column;

        /**
          * The column instance variable is used to remember how many
          * spaces remain to be inserted at the current input position.
          */
        int insert_spaces;

        /**
          * The default constructor.  Do not use.
          */
        input_filter_expand();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        input_filter_expand(const input_filter_expand &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        input_filter_expand &operator=(const input_filter_expand &rhs);
    };
}

#endif // LIBUNCIA_INPUT_EXPAND_H
