//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_INPUT_FILTER_GUNZIP_H
#define LIBUNCIA_INPUT_FILTER_GUNZIP_H

#include <zlib.h>

#include <libuncia/input/filter.h>

namespace libuncia
{
    /**
      * The input_filter_gunzip class is used to represent an input
      * filter which decompresses its input, which is required to be in
      * GNI Zip format.
      */
    class input_filter_gunzip:
        public input_filter
    {
    public:
        /**
          * The destructor.
          */
        virtual ~input_filter_gunzip();

    private:
        /**
          * The constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param deeper
          *     where to obtain the input to this filter.
          */
        input_filter_gunzip(const pointer &deeper);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper
          *     where to obtain the input to this filter.
          */
        static pointer create(const pointer &deeper);

    protected:
        // See base class for documentation.
        ssize_t read(void *data, size_t size);

    private:
        z_stream stream;
        bool z_eof;
        uLong crc;
        long pos;
        Byte *buf;
        std::string filename;

        void zlib_fatal_error(int err);
        long getLong();
        void read_header();
        unsigned char read_one_byte();

        /**
          * The default constructor.  Do not use.
          */
        input_filter_gunzip();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        input_filter_gunzip(const input_filter_gunzip &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        input_filter_gunzip &operator=(const input_filter_gunzip &rhs);
    };
}

#endif // LIBUNCIA_INPUT_FILTER_GUNZIP_H
