//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_INPUT_HEAD_H
#define LIBUNCIA_INPUT_HEAD_H

#include <libuncia/input/filter.h>

namespace libuncia
{
    /**
      * The input_filter_head class is used to represent and input
      * filter which limits the number of lines it passes through to the
      * first N lines.
      */
    class input_filter_head:
        public input_filter
    {
    public:
        /**
          * The destructor.
          */
        virtual ~input_filter_head();

    private:
        /**
          * The constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param deeper
          *     the input source to be filtered.
          * @paran limit
          *     the maximum number of lines to pass through.
          */
        input_filter_head(const pointer &deeper, long limit);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper
          *     the input source to be filtered.
          * @paran limit
          *     the maximum number of lines to pass through.
          */
        static pointer create(const pointer &deeper, long limit = 10);

    protected:
        // See base class for documentation.
        ssize_t read(void *data, size_t size);

    private:
        /**
          * The limit instance variable is used to remember the maximum
          * number of lines to pass through.
          */
        long limit;

        /**
          * The default constructor.  Do not use.
          */
        input_filter_head();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        input_filter_head(const input_filter_head &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        input_filter_head &operator=(const input_filter_head &rhs);
    };
}

#endif // LIBUNCIA_INPUT_HEAD_H
