//
//      uncia - a big cat
//      Copyright (C) 2008 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#include <libuncia/input/filter/number_lines.h>


libuncia::input_filter_number_lines::~input_filter_number_lines()
{
}


libuncia::input_filter_number_lines::input_filter_number_lines(
        const pointer &a_deeper, bool a_blank) :
    input_filter(a_deeper),
    blank(a_blank),
    line_number(0),
    start_of_line(true)
{
}


libuncia::input_filter_number_lines::pointer
libuncia::input_filter_number_lines::create(const pointer &a_deeper,
    bool a_blank)
{
    return pointer(new input_filter_number_lines(a_deeper, a_blank));
}


ssize_t
libuncia::input_filter_number_lines::read(void *data, size_t size)
{
    char *start = (char *)data;
    char *cp = start;
    char *end = start + size;
    while (cp < end)
    {
        char c = 0;
        if (!deeper_get(c))
            break;
        if (start_of_line)
        {
            start_of_line = false;
            if  (c != '\n' || blank)
            {
                deeper_putback(c);
                deeper_putback('\t');

                ++line_number;
                char buf[20];
                snprintf(buf, sizeof(buf), "%6ld", line_number);
                char *bp = buf + strlen(buf);
                while (bp > buf)
                {
                    --bp;
                    deeper_putback(*bp);
                }
                continue;
            }
        }
        *cp++ = c;
        if (c == '\n')
            start_of_line = true;
    }
    return (cp - start);
}
