//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_INPUT_NUMBER_LINES_H
#define LIBUNCIA_INPUT_NUMBER_LINES_H

#include <libuncia/input/filter.h>

namespace libuncia
{
    /**
      * The input_filter_number_lines class is used to represent the processing
      * necessary to prepend line numbers.
      */
    class input_filter_number_lines:
        public input_filter
    {
    public:
        /**
          * The destructor.
          */
        virtual ~input_filter_number_lines();

    private:
        /**
          * The constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param deeper
          *     The deepr input sours to read and filter.
          * @param blank
          *     whether or not to number blank lines
          */
        input_filter_number_lines(const pointer &deeper, bool blank = true);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper
          *     The deepr input sours to read and filter.
          * @param blank
          *     whether or not to number blank lines
          */
        static pointer create(const pointer &deeper, bool blank = true);

    protected:
        // See base class for documentation.
        ssize_t read(void *data, size_t size);

    private:
        /**
          * The blank instance variable is used to remember whether or
          * not blank lines are to be numbered.
          */
        bool blank;

        /**
          * The line_number instance variable is used to remember the
          * current line number.
          */
        long line_number;

        /**
          * The start_of_line instance variable is used to remember
          * whether or not the current read position is at the start of
          * a line.
          */
        bool start_of_line;

        /**
          * The default constructor.  Do not use.
          */
        input_filter_number_lines();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        input_filter_number_lines(const input_filter_number_lines &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        input_filter_number_lines &operator=(
            const input_filter_number_lines &rhs);
    };
}

#endif // LIBUNCIA_INPUT_NUMBER_LINES_H
