//
//      uncia - a big cat
//      Copyright (C) 2008 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#include <libuncia/input/filter/remove_prefix.h>


libuncia::input_filter_remove_prefix::~input_filter_remove_prefix()
{
}


libuncia::input_filter_remove_prefix::input_filter_remove_prefix(
        const pointer &a_deeper, const std::string &a_text) :
    input_filter(a_deeper),
    text(a_text),
    start_of_line(true)
{
}


libuncia::input::pointer
libuncia::input_filter_remove_prefix::create(const pointer &a_deeper,
    const std::string &a_text)
{
    if (a_text.empty())
        return a_deeper;
    return pointer(new input_filter_remove_prefix(a_deeper, a_text));
}


ssize_t
libuncia::input_filter_remove_prefix::read(void *data, size_t size)
{
    char *start = (char *)data;
    char *end = start + size;
    char *cp = start;
    while (cp < end)
    {
        if (start_of_line)
        {
            start_of_line = false;

            //
            // track how much of the prefix is matched
            //
            size_t pos = 0;
            while (pos < text.size())
            {
                char c = 0;
                if (!deeper_get(c))
                    break;
                if (c != text[pos])
                {
                    deeper_putback(c);
                    break;
                }
                ++pos;
            }

            //
            // If it all matched, swallow it and go again.
            //
            if (pos >= text.size())
                continue;

            //
            // Put it all back on the input.
            //
            while (pos > 0)
            {
                --pos;
                deeper_putback(text[pos]);
            }
        }

        if (!deeper_get(*cp))
            break;
        if (*cp == '\n')
            start_of_line = true;
        ++cp;
    }
    return (cp - start);
}
