//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_INPUT_REMOVE_PREFIX_H
#define LIBUNCIA_INPUT_REMOVE_PREFIX_H

#include <libuncia/input/filter.h>

namespace libuncia
{
    /**
      * The input_filter_remove_prefix class is used to represent a
      * filter which removes a constant prefix from input lines.
      */
    class input_filter_remove_prefix:
        public input_filter
    {
    public:
        /**
          * The destructor.
          */
        virtual ~input_filter_remove_prefix();

    private:
        /**
          * The constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param deeper
          *     The input to this filter.
          * @param text
          *     The constant string to remove from the start of each line.
          */
        input_filter_remove_prefix(const pointer &deeper,
            const std::string &text);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper
          *     The input to this filter.
          * @param text
          *     The constant string to remove from the start of each line.
          */
        static pointer create(const pointer &deeper, const std::string &text);

    protected:
        // See base class for documentation.
        ssize_t read(void *data, size_t size);

    private:
        /**
          * The text instance variable is used to remember the constant
          * string to remove from the start of each line.
          */
        std::string text;

        /**
          * The start_of_line instance variable is used to remember
          * whether or not the current input position is at the start of
          * a line.
          */
        bool start_of_line;

        /**
          * The default constructor.  Do not use.
          */
        input_filter_remove_prefix();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        input_filter_remove_prefix(const input_filter_remove_prefix &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        input_filter_remove_prefix &operator=(
            const input_filter_remove_prefix &rhs);
    };
}

#endif // LIBUNCIA_INPUT_REMOVE_PREFIX_H
