//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_INPUT_REVERSE_H
#define LIBUNCIA_INPUT_REVERSE_H

#include <libuncia/input/filter.h>

namespace libuncia
{
    /**
      * The input_filter_reverse class is used to represent a filter which
      * reverses the order of characters on each line.
      */
    class input_filter_reverse:
        public input_filter
    {
    public:
        /**
          * The destructor.
          */
        virtual ~input_filter_reverse();

    private:
        /**
          * The constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param deeper
          *     the source of data to be filtered
          */
        input_filter_reverse(const pointer &deeper);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper
          *     the source of data to be filtered
          */
        static pointer create(const pointer &deeper);

    protected:
        // See base class for documentation.
        ssize_t read(void *data, size_t size);

    private:
        typedef std::list<char> buffer_t;

        /**
          * The buffer instance variable is used to remember the input
          * line as it is read in.
          */
        buffer_t buffer;

        /**
          * The default constructor.  Do not use.
          */
        input_filter_reverse();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        input_filter_reverse(const input_filter_reverse &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        input_filter_reverse &operator=(const input_filter_reverse &rhs);
    };
}

#endif // LIBUNCIA_INPUT_REVERSE_H
