//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <libuncia/input/filter/reverse_lines.h>


libuncia::input_filter_reverse_lines::~input_filter_reverse_lines()
{
}


libuncia::input_filter_reverse_lines::input_filter_reverse_lines(
        const pointer &a_deeper) :
    input_filter(a_deeper),
    slurping(true)
{
}


libuncia::input::pointer
libuncia::input_filter_reverse_lines::create(const pointer &a_deeper)
{
    return pointer(new input_filter_reverse_lines(a_deeper));
}


ssize_t
libuncia::input_filter_reverse_lines::read(void *data, size_t size)
{
    if (slurping)
    {
        slurping = false;

        for (;;)
        {
            // read a line
            std::string line;
            for (;;)
            {
                char c;
                if (!deeper_get(c))
                {
                    if (!line.empty())
                    {
                        //
                        // Last line has no newline, so add one.  It's
                        // going to need it when it's the first line of
                        // the output.
                        //
                        line += '\n';
                    }
                    break;
                }
                line += c;
                if (c == '\n')
                    break;
            }

            //
            // can only get an empty line at end of file.
            //
            if (line.empty())
                break;

            //
            // stash the line for later
            //
            buffer.push_back(line);
        }
    }

    char *start = (char *)data;
    char *end = start + size;
    char *cp = start;
    while (cp < end)
    {
        if (buffer.empty())
            break;
        std::string line = buffer.back();

        //
        // If the line fits inside the buffer,
        // just copy it whole.
        //
        size_t avail = end - cp;
        if (line.size() <= avail)
        {
            memcpy(cp, line.c_str(), line.size());
            cp += line.size();
            buffer.pop_back();
            continue;
        }

        //
        // If the line doesn't fit, but some other lines did,
        // return short, and they can come back for more later.
        //
        if (cp > start)
            break;

        //
        // This is the difficult case, the line is larger than the whole
        // buffer the caller passed to us.  Break it up.
        //
        buffer.pop_back();
        memcpy(cp, line.c_str(), avail);
        cp += avail;
        buffer.push_back
        (
            std::string(line.c_str() + avail, line.size() - avail)
        );
        break;
    }
    return (cp - start);
}
