//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_INPUT_FILTER_REVERSE_LINES_H
#define LIBUNCIA_INPUT_FILTER_REVERSE_LINES_H

#include <libuncia/input/filter.h>

namespace libuncia
{
    /**
      * The input_filter_reverse_lines class is used to represent an
      * input filter which reverses the order of the lines in its input.
      * (It leaves the order of characters within the lines unchanged.)
      */
    class input_filter_reverse_lines:
        public input_filter
    {
    public:
        /**
          * The destructor.
          */
        virtual ~input_filter_reverse_lines();

    private:
        /**
          * The constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param deeper
          *     where to obtain the input to this filter.
          */
        input_filter_reverse_lines(const pointer &deeper);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper
          *     where to obtain the input to this filter.
          */
        static pointer create(const pointer &deeper);

    protected:
        // See base class for documentation.
        ssize_t read(void *data, size_t size);

    private:
        typedef std::list<std::string> buffer_t;

        /**
          * The buffer instance variable is used to remember the input
          * lines, in the order they appeared.
          */
        buffer_t buffer;

        /**
          * The slurping instance variable is used to remember whether
          * we are in slurping mode (true) or regugitating mode (false).
          */
        bool slurping;

        /**
          * The default constructor.  Do not use.
          */
        input_filter_reverse_lines();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        input_filter_reverse_lines(const input_filter_reverse_lines &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        input_filter_reverse_lines &operator=(
            const input_filter_reverse_lines &rhs);
    };
}

#endif // LIBUNCIA_INPUT_FILTER_REVERSE_LINES_H
