//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <climits>
#include <libuncia/input/filter/rot13.h>
#include <libuncia/sizeof.h>


libuncia::input_filter_rot13::~input_filter_rot13()
{
}


static char table[UCHAR_MAX + 1];


libuncia::input_filter_rot13::input_filter_rot13(const pointer &a_deeper) :
    input_filter(a_deeper)
{
    if (!table['a'])
    {
        for (size_t j = 0; j < SIZEOF(table); ++j)
        {
            // Note: http://hea-www.harvard.edu/~fine/Tech/rot13.html
            // makes interesting reading.
            unsigned char c = j;
            if (c >= 'A' && c <= 'Z')
                c = ((c + (13 - 'A')) % 26) + 'A';
            else if (c >= 'a' && c <= 'z')
                c = ((c + (13 - 'a')) % 26) + 'a';
            table[j] = c;
        }
    }
}


libuncia::input::pointer
libuncia::input_filter_rot13::create(const pointer &a_deeper)
{
    return pointer(new input_filter_rot13(a_deeper));
}


ssize_t
libuncia::input_filter_rot13::read(void *data, size_t size)
{
    char *start = (char *)data;
    char *end = start + size;
    char *cp = start;
    while (cp < end)
    {
        char c = 0;
        if (!deeper_get(c))
            break;
        *cp++ = table[(unsigned char)c];
    }
    return (cp - start);
}
