//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <libuncia/input/filter/sort.h>


libuncia::input_filter_sort::~input_filter_sort()
{
}


libuncia::input_filter_sort::input_filter_sort(const pointer &a_deeper) :
    input_filter(a_deeper),
    slurping(true)
{
}


libuncia::input::pointer
libuncia::input_filter_sort::create(const pointer &a_deeper)
{
    return pointer(new input_filter_sort(a_deeper));
}


ssize_t
libuncia::input_filter_sort::read(void *data, size_t size)
{
    if (slurping)
    {
        for (;;)
        {
            std::string line;
            if (!deeper_getline(line))
                break;
            lines.push_back(line);
        }
        slurping = false;

        //
        // Sort all the lines of the file.
        //
        std::sort(lines.begin(), lines.end());
    }

    char *start = (char *)data;
    char *end = start + size;
    char *cp = start;
    while (cp < end)
    {
        if (lines.empty())
            break;
        std::string line = lines.front();
        lines.pop_front();
        size_t avail = end - cp;
        if (line.size() >= avail)
        {
            //
            // The >= in the above comparison is because we need room
            // for the newline, too.
            //
            if (cp == start)
            {
                memcpy(cp, line.c_str(), avail);
                cp += avail;
                lines.push_front(line.substr(avail, line.size() - avail));
            }
            break;
        }
        memcpy(cp, line.c_str(), line.size());
        cp += line.size();
        *cp++ = '\n';
    }
    return (cp - start);
}
