//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_INPUT_FILTER_SORT_H
#define LIBUNCIA_INPUT_FILTER_SORT_H

#include <deque>
#include <libuncia/input/filter.h>
#include <libuncia/istream.h>

namespace libuncia
{
    /**
      * The input_filter_sort class is used to represent an input filter
      * which sorts its input lines (using strcmp, or equivalent).
      */
    class input_filter_sort:
        public input_filter
    {
    public:
        /**
          * The destructor.
          */
        virtual ~input_filter_sort();

    private:
        /**
          * The constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param deeper
          *     where to obtain the input to this filter.
          */
        input_filter_sort(const pointer &deeper);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper
          *     where to obtain the input to this filter.
          */
        static pointer create(const pointer &deeper);

    protected:
        // See base class for documentation.
        ssize_t read(void *data, size_t size);

    private:
        /**
          * The slurping instance variable is used to remember whether
          * or not we are still reading inout lines and stashing them
          * away in the #lines vector.
          */
        bool slurping;

        typedef std::deque<std::string> lines_t;

        /**
          * The lines instance variable is used to remember all the
          * lines in the input file.
          */
        lines_t lines;

        /**
          * The default constructor.  Do not use.
          */
        input_filter_sort();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        input_filter_sort(const input_filter_sort &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        input_filter_sort &operator=(const input_filter_sort &rhs);
    };
}

#endif // LIBUNCIA_INPUT_FILTER_SORT_H
