//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <libuncia/input/null.h>
#include <libuncia/input/filter/tail.h>


libuncia::input_filter_tail::~input_filter_tail()
{
}


libuncia::input_filter_tail::input_filter_tail(const pointer &a_deeper,
        long a_limit) :
    input_filter(a_deeper),
    limit(a_limit),
    slurping(true)
{
}


libuncia::input::pointer
libuncia::input_filter_tail::create(const pointer &a_deeper, long a_limit)
{
    if (a_limit <= 0)
        return libuncia::input_null::create();
    return pointer(new input_filter_tail(a_deeper, a_limit));
}


ssize_t
libuncia::input_filter_tail::read(void *data, size_t size)
{
    if (slurping)
    {
        //
        // Read the file contents, stashing the lines away as they are
        // read.  We only ever keep the last few.
        //
        for (;;)
        {
            std::string line;
            for (;;)
            {
                char c = 0;
                if (!deeper_get(c))
                {
                    if (!line.empty())
                        line += '\n';
                    break;
                }
                line += c;
                if (c == '\n')
                    break;
            }
            if (line.empty())
                break;

            buffer.push_back(line);
            if (buffer.size() > limit)
                buffer.pop_front();
        }
        slurping = false;
    }

    char *start = (char *)data;
    char *end = start + size;
    char *cp = start;
    while (cp < end)
    {
        if (buffer.empty())
            break;
        std::string line = buffer.front();
        buffer.pop_front();
        if (cp + line.size() > end)
        {
            size_t partial = end - cp;
            memcpy(cp, line.c_str(), partial);
            cp += partial;
            buffer.push_front(line.substr(partial, line.size() - partial));
            break;
        }
        memcpy(cp, line.c_str(), line.size());
        cp += line.size();
    }
    return (cp - start);
}
