//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_INPUT_TAIL_H
#define LIBUNCIA_INPUT_TAIL_H

#include <deque>

#include <libuncia/input/filter.h>

namespace libuncia
{
    /**
      * The input_filter_tail class is used to represent an input filter which
      * limits the data to the last N lines.
      */
    class input_filter_tail:
        public input_filter
    {
    public:
        /**
          * The destructor.
          */
        virtual ~input_filter_tail();

    private:
        /**
          * The constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param deeper
          *     where to get the data to be filtered.
          * @param limit
          *     the maximum number of lines to pass.
          */
        input_filter_tail(const pointer &deeper, long limit = 10);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper
          *     where to get the data to be filtered.
          * @param limit
          *     the maximum number of lines to pass.
          */
        static pointer create(const pointer &deeper, long limit = 10);

    protected:
        // See base class for documentation.
        ssize_t read(void *data, size_t size);

    private:
        /**
          * The limit instance variable is used to remember the maximum
          * number of lines to pass.
          */
        size_t limit;

        typedef std::deque<std::string> buffer_t;

        /**
          * The buffer instance variable is used to remember the lines
          * read from the input.
          */
        buffer_t buffer;

        /**
          * The slurping instance variable is used to remember wheter we
          * are in the slurping phase or the regurgitating phase.
          */
        bool slurping;

        /**
          * The default constructor.  Do not use.
          */
        input_filter_tail();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        input_filter_tail(const input_filter_tail &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        input_filter_tail &operator=(const input_filter_tail &rhs);
    };
}

#endif // LIBUNCIA_INPUT_TAIL_H
