//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <libuncia/input/filter/unique.h>


libuncia::input_filter_unique::~input_filter_unique()
{
}


libuncia::input_filter_unique::input_filter_unique(const pointer &a_deeper,
        bool a_include_count) :
    input_filter(a_deeper),
    include_count(a_include_count),
    repeat_count(0)
{
}


libuncia::input::pointer
libuncia::input_filter_unique::create(const pointer &a_deeper,
    bool a_include_count)
{
    return pointer(new input_filter_unique(a_deeper, a_include_count));
}


void
libuncia::input_filter_unique::emit_previous_line()
{
    if (repeat_count < 1)
        return;
    if (include_count)
    {
        char buf[20];
        snprintf(buf, sizeof(buf), "%7ld ", long(repeat_count));
        emit = buf + previous_line;
    }
    else
        emit = previous_line;
    emit += '\n';
    previous_line.clear();
    repeat_count = 0;
}


ssize_t
libuncia::input_filter_unique::read(void *data, size_t size)
{
    char *start = (char *)data;
    char *end = start + size;
    char *cp = start;
    while (cp < end)
    {
        if (!emit.empty())
        {
            size_t avail = end - cp;
            if (emit.size() > avail)
            {
                if (cp == start)
                {
                    memcpy(cp, emit.c_str(), avail);
                    cp += avail;
                    emit = emit.substr(avail, emit.size() - avail);
                }
                break;
            }
            memcpy(cp, emit.c_str(), emit.size());
            cp += emit.size();
            emit.clear();
            continue;
        }

        std::string line;
        if (!deeper_getline(line))
        {
            if (repeat_count > 0)
            {
                emit_previous_line();
                continue;
            }
            break;
        }
        if (!repeat_count)
        {
            previous_line = line;
            repeat_count = 1;
            continue;
        }
        if (line == previous_line)
        {
            ++repeat_count;
            continue;
        }
        emit_previous_line();
        previous_line = line;
        repeat_count = 1;
    }
    return (cp - start);
}
