//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_INPUT_FILTER_UUDECODE_H
#define LIBUNCIA_INPUT_FILTER_UUDECODE_H

#include <libuncia/input/filter.h>
#include <libuncia/istream.h>

namespace libuncia
{
    /**
      * The input_filter_uudecode class is used to represent an input
      * filter which uudecodes its input.
      */
    class input_filter_uudecode:
        public input_filter
    {
    public:
        /**
          * The destructor.
          */
        virtual ~input_filter_uudecode();

    private:
        /**
          * The constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param deeper
          *     where to obtain the input to this filter.
          */
        input_filter_uudecode(const pointer &deeper);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper
          *     where to obtain the input to this filter.
          */
        static pointer create(const pointer &deeper);

    protected:
        // See base class for documentation.
        ssize_t read(void *data, size_t size);

    private:
        long pos;
        int state;
        char etab[64];
        unsigned char itab[UCHAR_MAX + 1];
        int checksum;
        int padding;

        /**
          * The default constructor.  Do not use.
          */
        input_filter_uudecode();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        input_filter_uudecode(const input_filter_uudecode &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        input_filter_uudecode &operator=(const input_filter_uudecode &rhs);
    };
}

#endif // LIBUNCIA_INPUT_FILTER_UUDECODE_H
