//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_INPUT_ISTREAM_H
#define LIBUNCIA_INPUT_ISTREAM_H

#include <iostream>
#include <libuncia/istream.h>

namespace libuncia
{
    /**
      * The input_istream class is used to represent an input filter
      * which can read from a std::istream object.
      */
    class input_istream:
        public input
    {
    public:
        /**
          * The destructor.
          */
        virtual ~input_istream();

    private:
        /**
          * The constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param deeper
          *     where to obtain the input to this filter.  Its lifetime
          *     <b>must</b> exceed the lifetime of thos object.
          */
        input_istream(std::istream &deeper);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper
          *     where to obtain the input to this filter.
          */
        static pointer create(std::istream &deeper);

    protected:
        // See base class for documentation.
        ssize_t read(void *data, size_t size);

        // See base class for documentation.
        std::string filename_get() const;

    private:
        /**
          * The deeper instance variable is used to remember where to
          * obtain the input to this filter.
          */
        std::istream &deeper;

        /**
          * The default constructor.  Do not use.
          */
        input_istream();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        input_istream(const input_istream &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        input_istream &operator=(const input_istream &rhs);
    };
}

#endif // LIBUNCIA_INPUT_ISTREAM_H
