//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_INPUT_NULL_H
#define LIBUNCIA_INPUT_NULL_H

#include <libuncia/input.h>

namespace libuncia
{
    /**
      * The input_null class is used to represent the processing
      * required to emulate a null (empty) input stream.
      */
    class input_null:
        public input
    {
    public:
        /**
          * The destructor.
          */
        virtual ~input_null();

    private:
        /**
          * The default constructor.  It is private on purpose, use the
          * #create class method instead.
          */
        input_null();

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          */
        static pointer create();

    protected:
        // See base class for documentation.
        ssize_t read(void *data, size_t size);

        // See base class for documentation.
        std::string filename_get() const;

    private:
        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        input_null(const input_null &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        input_null &operator=(const input_null &rhs);
    };
}

#endif // LIBUNCIA_INPUT_NULL_H
