//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <libuncia/input/paste.h>


libuncia::input_paste::~input_paste()
{
}


libuncia::input_paste::input_paste(const pointer &a_deeper1,
        const std::string &a_delimiter, const pointer &a_deeper2) :
    deeper1(a_deeper1),
    delimiter(a_delimiter),
    deeper2(a_deeper2)
{
}


libuncia::input::pointer
libuncia::input_paste::create(const pointer &a_deeper1,
    const pointer &a_deeper2)
{
    return pointer(new input_paste(a_deeper1, "\t", a_deeper2));
}


libuncia::input::pointer
libuncia::input_paste::create(const std::string &a_delimiter,
    const pointer &a_deeper1, const pointer &a_deeper2)
{
    return pointer(new input_paste(a_deeper1, a_delimiter, a_deeper2));
}


ssize_t
libuncia::input_paste::read(void *data, size_t size)
{
    char *start = (char *)data;
    char *end = start + size;
    char *cp = start;
    while (cp < end)
    {
        if (buffer.empty())
        {
            std::string l1;
            std::string l2;
            bool ok1 = deeper1.getline(l1);
            bool ok2 = deeper2.getline(l2);
            if (!ok1 && !ok2)
                break;
            buffer = l1 + delimiter + l2 + "\n";
        }
        size_t avail = end - cp;
        if (buffer.size() <= avail)
        {
            memcpy(cp, buffer.c_str(), buffer.size());
            cp += buffer.size();
            buffer.clear();
        }
        else
        {
            //
            // if the line doesn't fit, but previous lines did,
            // stop early and come back for the rest later.
            //
            if (cp > start)
                break;

            //
            // This is the difficult case, we have to break up the line.
            //
            std::string part1 = buffer.substr(0, avail);
            std::string part2 = buffer.substr(avail, buffer.size() - avail);
            memcpy(cp, part1.c_str(), part1.size());
            cp += sizeof(part1);
            buffer = part2;
            break;
        }
    }
    return (cp - start);
}


std::string
libuncia::input_paste::filename_get()
    const
{
    return "-paste " + deeper1.filename_get() + " " + deeper2.filename_get();
}
