//
//      uncia - a big cat
//      Copyright (C) 2008 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#include <boost/algorithm/string/predicate.hpp>
#include <string>

#include <libuncia/error_handler.h>
#include <libuncia/input/stdin.h>


libuncia::input_stdin::~input_stdin()
{
}


libuncia::input_stdin::input_stdin() :
    filename("standard input")
{
    //
    // On Linx we are able to grope out file descriptors to locate the
    // information withing the /proc file system.  This reflects the
    // kernel's information about our file descriptors.  Try to use that
    // information to locate the filename associated with stdout, so
    // that we can give much better error messages.
    //
    char buf[2000];
    int n = readlink("/dev/stdin", buf, sizeof(buf));
    if (n > 0)
    {
        std::string l1(buf, n);
        if (boost::starts_with(l1, "/proc/self/fd/"))
        {
            n = readlink(l1.c_str(), buf, sizeof(buf));
            if (n > 0)
            {
                std::string l2(buf, n);
                if
                (
                    !boost::starts_with(l2, "/proc/")
                &&
                    !boost::starts_with(l2, "/dev/")
                &&
                    !boost::starts_with(l2, "pipe:")
                )
                    filename = l2;
            }
        }
    }
}


libuncia::input_stdin::pointer
libuncia::input_stdin::create()
{
    return pointer(new input_stdin());
}


ssize_t
libuncia::input_stdin::read(void *data, size_t size)
{
    ssize_t ret = ::read(0, data, size);
    if (ret < 0)
    {
        error_handler_get()->fatal_error_errno("read %s", filename.c_str());
    }
    return ret;
}


std::string
libuncia::input_stdin::filename_get()
    const
{
    return filename;
}
