//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_INPUT_STDIN_H
#define LIBUNCIA_INPUT_STDIN_H

#include <libuncia/input.h>

namespace libuncia
{
    /**
      * The input_stdin class is used to represent the starndard input
      * as an data source.
      */
    class input_stdin:
        public input
    {
    public:
        /**
          * The destructor.
          */
        virtual ~input_stdin();

    private:
        /**
          * The default constructor.
          */
        input_stdin();

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          */
        static pointer create();

    protected:
        // See base class for documentation.
        ssize_t read(void *data, size_t size);

        // See base class for documentation.
        std::string filename_get() const;

    private:
        /**
          * The filename instance variable is used to remember the name
          * of this input, to improve error messages.
          */
        std::string filename;

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        input_stdin(const input_stdin &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        input_stdin &operator=(const input_stdin &rhs);
    };
}

#endif // LIBUNCIA_INPUT_STDIN_H
