//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <libuncia/istream.h>
#include <libuncia/istreambuf.h>


libuncia::istream::~istream()
{
    delete rdbuf();
}


libuncia::istream::istream(const libuncia::input::pointer &rhs) :
    std::istream(libuncia::istreambuf::create(rhs)),
    raw(rhs)
{
}


void
libuncia::istream::fatal_error(const char *fmt, ...)
{
    va_list ap;
    va_start(ap, fmt);
    fatal_error_v(fmt, ap);
    va_end(ap);
}


void
libuncia::istream::fatal_error_v(const char *fmt, va_list ap)
{
    raw->fatal_error_v(fmt, ap);
}


std::string
libuncia::istream::filename_get()
    const
{
    return raw->filename_get();
}


bool
libuncia::istream::getline(std::string &result)
{
    static char *buffer;
    static size_t bufsiz_max;
    size_t bufsiz = 0;
    for (;;)
    {
        char c = 0;
        if (!get(c))
        {
            if (bufsiz == 0)
                return false;
            c = '\n';
        }
        if (c == '\n')
        {
            result = std::string(buffer, bufsiz);
            return true;
        }

        if (bufsiz >= bufsiz_max)
        {
            size_t new_bufsiz_max = bufsiz_max * 2 + 64;
            char *new_buffer = new char [new_bufsiz_max];
            memcpy(new_buffer, buffer, bufsiz);
            delete [] buffer;
            buffer = new_buffer;
            bufsiz_max = new_bufsiz_max;
        }
        buffer[bufsiz++] = c;
    }
}
