//
//      uncia - a big cat
//      Copyright (C) 2008 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#include <libuncia/istreambuf.h>


libuncia::istreambuf::~istreambuf()
{
    delete [] buffer_base;
    buffer_base = 0;
    buffer_size = 0;
}


libuncia::istreambuf::istreambuf(const input::pointer &rhs) :
    deeper(rhs),
    buffer_size(1 << 14),
    buffer_base(new char [buffer_size])
{
    char *buffer_end = buffer_base + buffer_size;
    setg(buffer_base, buffer_end, buffer_end);
}


std::streambuf *
libuncia::istreambuf::create(const input::pointer &rhs)
{
    return new istreambuf(rhs);
}


std::streambuf::int_type
libuncia::istreambuf::underflow()
{
    // http://publib.boulder.ibm.com/infocenter/comphelp/v9v111/index.jsp?
    //     topic=/com.ibm.xlcpp9.aix.doc/standlib/header_streambuf.htm
    //
    // The underflow method endeavors to extract the current element
    // c from the input stream, without advancing the current stream
    // position, and return it as traits_type::to_int_type(c). It can do
    // so in various ways:
    //
    // * If a read position is available, c is the element stored in the
    //   read position.
    // * It can make a read position available by allocating new or
    //   additional storage for the input buffer, then reading in, from
    //   some external source, one or more elements.
    //
    // If the function cannot succeed, it returns traits_type::eof(), or
    // throws an exception. Otherwise, it returns the current element
    // in the input stream, converted as described above. The default
    // behavior is to return traits_type::eof().
    //
    if (gptr() >= egptr())
    {
        int nbytes = deeper->read(buffer_base, buffer_size);
        if (nbytes <= 0)
            return traits_type::eof();
        setg(buffer_base, buffer_base, buffer_base + nbytes);
    }

    char_type c = *gptr();
    // do not gbump(1)
    return traits_type::to_int_type(c);
}
