//
//      uncia - a big cat
//      Copyright (C) 2008 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_ISTREAMBUF_H
#define LIBUNCIA_ISTREAMBUF_H

#include <iostream>
#include <libuncia/input.h>

namespace libuncia
{
    /**
      * The istreambuf class is used to represent the adaptations
      * necessary to present an libuncia::input as a C++ standard
      * streambuf.
      */
    class istreambuf:
        public std::streambuf
    {
    public:
        /**
          * The destructor.
          */
        virtual ~istreambuf();

    private:
        /**
          * The constructor.  It is private on purpose, use the #create
          * class method instead.
          *
          * @param rhs
          *     The input source to use.
          */
        istreambuf(const libuncia::input::pointer &rhs);

    public:
        /**
          * The create class method mat be used to create a new dynamically
          * allocated instance of this class.
          *
          * @param rhs
          *     The input source to use.
          *
          * @note
          *     The expected usage is to be immediately handed to a C++
          *     standard library istream constructor, e.g.
          *     <pre>
          *     istream fred(libuncia::istreambuf::create(blah));
          *     </pre>
          */
        static std::streambuf *create(const libuncia::input::pointer &rhs);

    protected:
        // See base class for documentation.
        int_type underflow();

    private:
        /**
          * The deeper instance variable is used to remember the actual
          * source of data for this input streambuf.
          */
        input::pointer deeper;

        /**
          * The buffer_size instance variable is used to remember how
          * many bytes of space were allocated to the buffer pointed to
          * tby the #buffer_base instance variable.
          */
        size_t buffer_size;

        /**
          * The buffer_base instance variable is used to remember the
          * base address of a dynamically allocated array of characters
          * used to buffer the input.
          */
        char *buffer_base;

        /**
          * The default constructor.  Do not use.
          */
        istreambuf();

        /**
          * The copy constructor.  Do not use.
          */
        istreambuf(const istreambuf &);

        /**
          * The assignment operator.  Do not use.
          */
        istreambuf &operator=(const istreambuf &);
    };
}

#endif // LIBUNCIA_ISTREAMBUF_H
