//
//      uncia - a big cat
//      Copyright (C) 2008 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_OSTREAM_H
#define LIBUNCIA_OSTREAM_H

#include <iostream>

#include <libuncia/ostreambuf.h>

namespace libuncia
{
    /**
      * The ostream class is used to represent an adapter from C++
      * standard ostream to an libuncia::output.
      */
    class ostream:
        public std::ostream
    {
    public:
        /**
          * The destructor.
          */
        virtual ~ostream();

        /**
          * The constructor.
          *
          * @param rhs
          *     The libuncia::ouptput to deliver data to.
          */
        ostream(const output::pointer &rhs);

        /**
          * The fatal_error method may be used to report errors about
          * this output stream.  Where possible, the name of the output
          * will be inserted into the error message.
          *
          * @param fmt
          *     The text of the error message, with embedeed formatting
          *     controls which describe the remaining arguments.  See
          *     printf(3) for more information.
          */
        void fatal_error(const char *fmt, ...)
                                                        FORMAT_PRINTF(2, 3);

        /**
          * The fatal_error method may be used to report errors about
          * this output stream.  Where possible, the name of the output
          * will be inserted into the error message.
          *
          * @param fmt
          *     The text of the error message, with embedeed formatting
          *     controls which describe the remaining arguments.  See
          *     vprintf(3) for more information.
          * @param ap
          *     Where to obtain the remaining arguments.
          */
        void fatal_error_v(const char *fmt, va_list ap)
                                                        FORMAT_PRINTF(2, 0);

        /**
          * The filename_get method may be used to obtain the name of
          * the output.  This is used by the fatal_error methods to
          * improve error messages.
          */
        std::string filename_get() const;

    private:
        /**
          * The raw instance variable is used to remmember the unbuffered
          * stream this output will be delivered to.
          */
        libuncia::output::pointer raw;

        /**
          * The default constructor.  Do not use.
          */
        ostream();

        /**
          * The copy constructor.  Do not use.
          */
        ostream(const ostream &);

        /**
          * The assignment operator.  Do not use.
          */
        ostream &operator=(const ostream &);
    };
}

#endif // LIBUNCIA_OSTREAM_H
