//
//      uncia - a big cat
//      Copyright (C) 2008 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#include <libuncia/ostreambuf.h>


libuncia::ostreambuf::~ostreambuf()
{
    sync();
    setp(0, 0);
    delete [] buffer_base;
    buffer_size = 0;
    buffer_base = 0;
}


libuncia::ostreambuf::ostreambuf(const libuncia::output::pointer &rhs) :
    deeper(rhs),
    buffer_size(1 << 14),
    buffer_base(new char [buffer_size])
{
    setp(buffer_base, buffer_base + buffer_size);
}


std::streambuf *
libuncia::ostreambuf::create(const libuncia::output::pointer &rhs)
{
    return new libuncia::ostreambuf(rhs);
}


libuncia::ostreambuf::int_type
libuncia::ostreambuf::overflow(int_type c)
{
    //
    // Write the buffered data to the deeper stream.
    //
    {
        size_t nbytes = pptr() - pbase();
        if (nbytes > 0)
        {
            int err = deeper->write(pbase(), nbytes);
            if (err < 0)
                return traits_type::eof();
        }
        setp(buffer_base, buffer_base + buffer_size);
    }

    //
    // If a character was given, add it to the buffer.
    //
    if (c != traits_type::eof())
    {
        // now there is enough room
        *pptr() = traits_type::to_char_type(c);
        pbump(1);
    }

    //
    // Report success
    //
    return traits_type::not_eof(c);
}


int
libuncia::ostreambuf::sync()
{
    //
    // Make sure all the data as been written.
    //
    {
        int_type err = overflow(traits_type::eof());
        if (err == traits_type::eof())
            return -1;
    }

    //
    // flush the deeper output, too.
    //
    {
        int err = deeper->sync();
        if (err < 0)
            return -1;
    }

    //
    // report success
    //
    return 0;
}
