//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_OUTPUT_FILE_H
#define LIBUNCIA_OUTPUT_FILE_H

#include <string>
#include <libuncia/output.h>

namespace libuncia
{
    /**
      * The output_file class is used to represent the processing
      * necessary to write data to a file.
      */
    class output_file:
        public output
    {
    public:
        /**
          * The destructor.
          */
        virtual ~output_file();

    private:
        /**
          * The constructor.  It is private on purpose, use the #create
          * class method instead.
          *
          * @param filename
          *     The name of the file to be written.
          */
        output_file(const std::string &filename);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param filename
          *     The name of the file to be written.
          */
        static pointer create(const std::string &filename);

    protected:
        // See base class for documentation.
        ssize_t write(const void *data, size_t size);

        // See base class for documentation.
        int sync();

        // See base class for documentation.
        std::string filename_get() const;

    private:
        /**
          * The filename instance variable is used to remember the name
          * of the file being written.
          */
        std::string filename;

        /**
          * The fd instance variable is used to remember the number of
          * the open file descriptor for this file.
          */
        int fd;

        /**
          * The default constructor.  Do not use.
          */
        output_file();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        output_file(const output_file &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        output_file &operator=(const output_file &rhs);
    };
}

#endif // LIBUNCIA_OUTPUT_FILE_H
