//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_OUTPUT_FILTER_H
#define LIBUNCIA_OUTPUT_FILTER_H

#include <libuncia/ostream.h>
#include <libuncia/output.h>

namespace libuncia
{
    /**
      * The output_filter abstract base class is used to represent a
      * gneric output filter.
      */
    class output_filter:
        public output
    {
    public:
        /**
          * The destructor.
          */
        virtual ~output_filter();

    protected:
        /**
          * The constructor.
          *
          * @param deeper
          *     where to send the output of this filter.
          */
        output_filter(const pointer &deeper);

    protected:
        // See base class for documentation.
        int sync();

        // See base class for documentation.
        std::string filename_get() const;

        /**
          * The deeper putc method may be used to add a charactre to the
          * deeper output.
          *
          * @param c
          *     the character to be written
          */
        void
        deeper_putc(char c)
        {
            deeper << c;
        }

        /**
          * The deeper_puts method may be used to write strings to the
          * deeper output.
          *
          * @param text
          *     the string to be written
          */
        void
        deeper_puts(const char *text)
        {
            deeper << text;
        }

        /**
          * The deeper_puts method may be used to write strings to the
          * deeper output.
          *
          * @param text
          *     the string to be written
          */
        void
        deeper_puts(const std::string &text)
        {
            deeper << text;
        }

        /**
          * The deeper_write method may be used to write opque byte data
          * to the deeper output.
          *
          * @param dats
          *     pointer to the base of the data to be written
          * @param size
          *     number of bytes of data to be written
          */
        void
        deeper_write(const void *data, size_t size)
        {
            deeper.write((const char *)data, size);
        }

    private:
        /**
          * The deeper instance variable is used to remember where to
          * send the output of this filter.
          */
        libuncia::ostream deeper;

        /**
          * The default constructor.  Do not use.
          */
        output_filter();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        output_filter(const output_filter &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        output_filter &operator=(const output_filter &rhs);
    };
}

#endif // LIBUNCIA_OUTPUT_FILTER_H
