//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <libuncia/output/filter/base64.h>


static char base64[] =
    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";


static inline char
map64(int n)
{
    return base64[n & 0x3F];
}


libuncia::output_filter_base64::~output_filter_base64()
{
    switch (residual_bits)
    {
    case 4:
        deeper_putc(map64(residual_value << 2));
        deeper_putc('=');
        output_column += 2;
        break;

    case 2:
        deeper_putc(map64(residual_value << 4));
        deeper_puts("==");
        output_column += 3;
        break;
    }
    if (output_column)
        deeper_putc('\n');
}


libuncia::output_filter_base64::output_filter_base64(const pointer &a_deeper) :
    output_filter(a_deeper),
    residual_value(0),
    residual_bits(0),
    output_column(0)
{
}


libuncia::output::pointer
libuncia::output_filter_base64::create(const pointer &a_deeper)
{
    return pointer(new output_filter_base64(a_deeper));
}


ssize_t
libuncia::output_filter_base64::write(const void *data, size_t size)
{
    const unsigned char *start = (const unsigned char *)data;
    const unsigned char *end = start + size;
    const unsigned char *cp = start;
    while (cp < end)
    {
        unsigned char c = *cp++;

        residual_value = (residual_value << 8) | c;
        residual_bits += 8;
        for (;;)
        {
            residual_bits -= 6;
            deeper_putc(map64(residual_value >> residual_bits));
            ++output_column;
            if (residual_bits == 0 && output_column > 70)
            {
                deeper_putc('\n');
                output_column = 0;
            }
            if (residual_bits < 6)
                break;
        }
    }
    return size;
}
