//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_OUTPUT_FILTER_CRLF_H
#define LIBUNCIA_OUTPUT_FILTER_CRLF_H

#include <libuncia/output/filter.h>

namespace libuncia
{
    /**
      * The output_filter_crlf class is used to represent an output filter
      * which replaces NL with CR LF.
      */
    class output_filter_crlf:
        public output_filter
    {
    public:
        /**
          * The destructor.
          */
        virtual ~output_filter_crlf();

    private:
        /**
          * The default constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param deeper
          *     where to write the output of this filter.
          */
        output_filter_crlf(const pointer &deeper);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper
          *     where to write the output of this filter.
          */
        static pointer create(const pointer &deeper);

    protected:
        // See base class for documentation.
        ssize_t write(const void *data, size_t size);

    private:
        /**
          * The prev_was_cr instance variable is used to remember
          * whether or not the last character sent to the deeper output
          * was a carriage return.  This is used to avoid inserting
          * redundant carriage returns.
          */
        bool prev_was_cr;

        /**
          * The default constructor.  Do not use.
          */
        output_filter_crlf();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        output_filter_crlf(const output_filter_crlf &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        output_filter_crlf &operator=(const output_filter_crlf &rhs);
    };
}

#endif // LIBUNCIA_OUTPUT_FILTER_CRLF_H
