//
//      uncia - a big cat
//      Copyright (C) 2008 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#include <cassert>
#include <libuncia/output/filter/hexdump.h>


libuncia::output_filter_hexdump::~output_filter_hexdump()
{
    dump();
}


libuncia::output_filter_hexdump::output_filter_hexdump(
        const pointer &a_deeper) :
    output_filter(a_deeper),
    position(0),
    address(0)
{
}


libuncia::output_filter_hexdump::pointer
libuncia::output_filter_hexdump::create(const pointer &a_deeper)
{
    return pointer(new output_filter_hexdump(a_deeper));
}


ssize_t
libuncia::output_filter_hexdump::write(const void *data, size_t size)
{
    const unsigned char *start = (const unsigned char *)data;
    const unsigned char *end = start + size;
    const unsigned char *cp = start;
    while (cp < end)
    {
        unsigned char c = *cp++;
        buffer[position++] = c;
        if (position >= sizeof(buffer))
            dump();
    }
    return size;
}


void
libuncia::output_filter_hexdump::dump()
{
    assert(position <= sizeof(buffer));
    if (!position)
        return;
    char tmp[20];
    snprintf(tmp, sizeof(tmp), "%04lX:", (unsigned long)address);
    deeper_puts(tmp);
    for (size_t j = 0; j < sizeof(buffer); ++j)
    {
        deeper_putc(' ');
        if (j < position)
        {
            snprintf(tmp, sizeof(tmp), "%02X", buffer[j]);
            deeper_puts(tmp);
        }
        else
            deeper_puts("  ");
    }
    deeper_puts("  ");
    for (size_t k = 0; k < position; ++k)
    {
        unsigned char c = 0x7F & buffer[k];
        if (isprint(c))
            deeper_putc(c);
        else
            deeper_putc('.');
    }
    deeper_putc('\n');
    address += position;
    position = 0;
}
