//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_OUTPUT_FILTER_HEXDUMP_H
#define LIBUNCIA_OUTPUT_FILTER_HEXDUMP_H

#include <libuncia/output/filter.h>

namespace libuncia
{
    /**
      * The output_filter_hexdump class is used to represent an output
      * filter which emits a hexadecimal listing of its input.
      */
    class output_filter_hexdump:
        public output_filter
    {
    public:
        /**
          * The destructor.
          */
        virtual ~output_filter_hexdump();

    private:
        /**
          * The default constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param deeper
          *     where to send the output of this filter.
          */
        output_filter_hexdump(const pointer &deeper);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper
          *     where to send the output of this filter.
          */
        static pointer create(const pointer &deeper);

    protected:
        // See base class for documentation.
        ssize_t write(const void *data, size_t size);

    private:
        /**
          * The buffer instance variable is used to remember the values
          * of the output line being constructed.
          */
        unsigned char buffer[16];

        /**
          * The position instance variable is used to remember the
          * current position withing the #buffer array.
          */
        size_t position;

        /**
          * The address instance variable is used to remember the
          * current output position of the start of the #buffer.  This
          * is used to insert the address at the start of the hexdump
          * line.
          */
        size_t address;

        /**
          * The dump method is used to write out the buffered data as a
          * hexdump, and rest the #position and advance the #address.
          */
        void dump();

        /**
          * The default constructor.  Do not use.
          */
        output_filter_hexdump();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        output_filter_hexdump(const output_filter_hexdump &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        output_filter_hexdump &operator=(const output_filter_hexdump &rhs);
    };
}

#endif // LIBUNCIA_OUTPUT_FILTER_HEXDUMP_H
