//
//      uncia - a big cat
//      Copyright (C) 2008 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#include <libuncia/output/filter/number_lines.h>


libuncia::output_filter_number_lines::~output_filter_number_lines()
{
}


libuncia::output_filter_number_lines::output_filter_number_lines(
        const pointer &a_deeper, bool a_blank) :
    output_filter(a_deeper),
    blank(a_blank),
    line_number(0),
    start_of_line(true)
{
}


libuncia::output::pointer
libuncia::output_filter_number_lines::create(const pointer &a_deeper,
    bool a_blank)
{
    return pointer(new libuncia::output_filter_number_lines(a_deeper, a_blank));
}


ssize_t
libuncia::output_filter_number_lines::write(const void *data, size_t size)
{
    const char *cp = (const char *)data;
    const char *ep = cp + size;
    while (cp < ep)
    {
        if (start_of_line)
        {
            if (*cp != '\n' || blank)
            {
                ++line_number;
                char buf[20];
                snprintf(buf, sizeof(buf), "%6ld\t", line_number);
                deeper_puts(buf);
            }
            start_of_line = false;
        }
        char c = *cp++;
        deeper_putc(c);
        if (c == '\n')
            start_of_line = true;
    }
    return size;
}
