//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_OUTPUT_FILTER_NUMBER_LINES_H
#define LIBUNCIA_OUTPUT_FILTER_NUMBER_LINES_H

#include <libuncia/output/filter.h>

namespace libuncia
{
    /**
      * The output_filter_number_lines class is used to represent the
      * processing required to number lines in a text file on output.
      */
    class output_filter_number_lines:
        public output_filter
    {
    public:
        /**
          * The destructor.
          */
        virtual ~output_filter_number_lines();

    private:
        /**
          * The constructor.
          * It is private on purpose, use the #create method instead.
          *
          * @param rhs
          *     The deeper output to be written to.
          * @param blank
          *     True if should number blank lines, false if should not.
          */
        output_filter_number_lines(const libuncia::output::pointer &rhs,
            bool blank = true);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param rhs
          *     The deeper output to be written to.
          * @param blank
          *     True if should number blank lines, false if should not.
          */
        static pointer create(const libuncia::output::pointer &rhs,
            bool blank = true);

    protected:
        // See base class for documentation.
        ssize_t write(const void *data, size_t size);

    private:
        /**
          * The blank instance variable is used to remember whether or
          * not to number blank lines.
          */
        bool blank;

        /**
          * The line_number instance variable is used to remember the
          * current line number of the output.
          */
        long line_number;

        /**
          * The start_of_line instance variable is used to remember
          * whether or not the current output position is at the start
          * of a line.
          */
        bool start_of_line;

        /**
          * The default constructor.  Do not use.
          */
        output_filter_number_lines();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        output_filter_number_lines(const output_filter_number_lines &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        output_filter_number_lines &operator=(
            const output_filter_number_lines &rhs);
    };
}

#endif // LIBUNCIA_OUTPUT_FILTER_NUMBER_LINES_H
