//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_OUTPUT_FILTER_PREFIX_H
#define LIBUNCIA_OUTPUT_FILTER_PREFIX_H

#include <libuncia/output/filter.h>

namespace libuncia
{
    /**
      * The output_filter_prefix class is used to represent a filter which adds
      * a constant prefix to each output line.
      */
    class output_filter_prefix:
        public output_filter
    {
    public:
        /**
          * The destructor.
          */
        virtual ~output_filter_prefix();

    private:
        /**
          * The constructor.
          * It is private on purpose, use the #create method instead.
          *
          * @param deeper
          *     The deeper output to be written to.
          * @param text
          *     The text to be prefixed to the start of each line.
          */
        output_filter_prefix(const libuncia::output::pointer &deeper,
            const std::string &text);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param rhs
          *     The deeper output to be written to.
          * @param text
          *     The text to be prefixed to the start of each line.
          */
        static pointer create(const libuncia::output::pointer &rhs,
            const std::string &text);

    protected:
        // See base class for documentation.
        ssize_t write(const void *data, size_t size);

    private:
        /**
          * The text instance variable is used to remember the text to
          * be inserted at the start of each line.
          */
        std::string text;

        /**
          * The start_of_line instance variable is used to remember
          * whether or not the current output position is at the start
          * of a line.
          */
        bool start_of_line;

        /**
          * The default constructor.  Do not use.
          */
        output_filter_prefix();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        output_filter_prefix(const output_filter_prefix &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        output_filter_prefix &operator=(const output_filter_prefix &rhs);
    };
}

#endif // LIBUNCIA_OUTPUT_FILTER_PREFIX_H
