//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_OUTPUT_FILTER_REVERSE_H
#define LIBUNCIA_OUTPUT_FILTER_REVERSE_H

#include <list>
#include <libuncia/output/filter.h>

namespace libuncia
{
    /**
      * The output_filter_reverse class is used to represent a filter which
      * reverses the order of characters on a line.
      */
    class output_filter_reverse:
        public output_filter
    {
    public:
        /**
          * The destructor.
          */
        virtual ~output_filter_reverse();

    private:
        /**
          * The constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param deeper
          *     where to write the output of this filter.
          */
        output_filter_reverse(const pointer &deeper);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper
          *     where to write the output of this filter.
          */
        static pointer create(const pointer &deeper);

    protected:
        // See base class for documentation.
        ssize_t write(const void *data, size_t size);

    private:
        typedef std::list<char> buffer_t;

        /**
          * The buffer instance variable is used to remember the
          * characters on the line, between newlines.
          */
        buffer_t buffer;

        /**
          * The default constructor.  Do not use.
          */
        output_filter_reverse();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        output_filter_reverse(const output_filter_reverse &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        output_filter_reverse &operator=(const output_filter_reverse &rhs);
    };
}

#endif // LIBUNCIA_OUTPUT_FILTER_REVERSE_H
