//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <climits>
#include <libuncia/output/filter/rot13.h>
#include <libuncia/sizeof.h>


static char table[UCHAR_MAX + 1];


libuncia::output_filter_rot13::~output_filter_rot13()
{
}


libuncia::output_filter_rot13::output_filter_rot13(const pointer &a_deeper) :
    output_filter(a_deeper)
{
    if (!table['a'])
    {
        for (size_t j = 0; j < SIZEOF(table); ++j)
        {
            // Note: http://hea-www.harvard.edu/~fine/Tech/rot13.html
            // makes interesting reading.
            unsigned char c = j;
            if (c >= 'A' && c <= 'Z')
                c = ((c + (13 - 'A')) % 26) + 'A';
            else if (c >= 'a' && c <= 'z')
                c = ((c + (13 - 'a')) % 26) + 'a';
            table[j] = c;
        }
    }
}


libuncia::output::pointer
libuncia::output_filter_rot13::create(const pointer &a_deeper)
{
    return pointer(new output_filter_rot13(a_deeper));
}


ssize_t
libuncia::output_filter_rot13::write(const void *data, size_t size)
{
    const char *start = (const char *)data;
    const char *end = start + size;
    const char *cp = start;
    while (cp < end)
    {
        unsigned char c = *cp++;
        c = table[c];
        deeper_putc(c);
    }
    return size;
}
