//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_OUTPUT_FILTER_SHOW_NONPRINTING_H
#define LIBUNCIA_OUTPUT_FILTER_SHOW_NONPRINTING_H

#include <libuncia/output/filter.h>

namespace libuncia
{
    /**
      * The output_filter_show_nonprinting class is used to represent an
      * output filter which shows non-printing characters (but leaves
      * tabs and newlines alone).  See cat -v for details.
      */
    class output_filter_show_nonprinting:
        public output_filter
    {
    public:
        /**
          * The destructor.
          */
        virtual ~output_filter_show_nonprinting();

    private:
        /**
          * The default constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param deeper
          *     where to send the filtered data.
          */
        output_filter_show_nonprinting(const pointer &deeper);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper
          *     where to send the filtered data.
          */
        static pointer create(const pointer &deeper);

    protected:
        // See base class for documentation.
        ssize_t write(const void *data, size_t size);

    private:
        /**
          * The default constructor.  Do not use.
          */
        output_filter_show_nonprinting();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        output_filter_show_nonprinting(
            const output_filter_show_nonprinting &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        output_filter_show_nonprinting &operator=(
            const output_filter_show_nonprinting &rhs);
    };
}

#endif // LIBUNCIA_OUTPUT_FILTER_SHOW_NONPRINTING_H
