//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <libuncia/output/filter/suffix_add.h>


libuncia::output_filter_suffix_add::~output_filter_suffix_add()
{
    if (column)
    {
        // last line not terminated
        deeper_puts(text);
    }
}


libuncia::output_filter_suffix_add::output_filter_suffix_add(
        const pointer &a_deeper, const std::string &a_text) :
    output_filter(a_deeper),
    text(a_text),
    column(0)
{
}


libuncia::output::pointer
libuncia::output_filter_suffix_add::create(const pointer &a_deeper,
    const std::string &a_text)
{
    if (a_text.empty())
        return a_deeper;
    return pointer(new output_filter_suffix_add(a_deeper, a_text));
}


ssize_t
libuncia::output_filter_suffix_add::write(const void *data, size_t size)
{
    const char *start = (const char *)data;
    const char *end = start + size;
    const char *cp = start;
    while (cp < end)
    {
        char c = *cp++;
        if (c == '\n')
        {
            deeper_puts(text);
            deeper_putc('\n');
            column = 0;
        }
        else
        {
            deeper_putc(c);
            ++column;
        }
    }
    return size;
}
