//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_OUTPUT_FILTER_SUFFIX_ADD_H
#define LIBUNCIA_OUTPUT_FILTER_SUFFIX_ADD_H

#include <libuncia/output/filter.h>

namespace libuncia
{
    /**
      * The output_filter_suffix_add class is used to represent an aoutput
      * filter which appends a constant suffix to each line.
      */
    class output_filter_suffix_add:
        public output_filter
    {
    public:
        /**
          * The destructor.
          */
        virtual ~output_filter_suffix_add();

    private:
        /**
          * The constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param deeper
          *     where to send the output of this filter
          * @param text
          *     what to add to the end of each line
          */
        output_filter_suffix_add(const pointer &deeper,
            const std::string &text);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper
          *     where to send the output of this filter
          * @param text
          *     what to add to the end of each line
          */
        static pointer create(const pointer &deeper, const std::string &text);

    protected:
        // See base class for documentation.
        ssize_t write(const void *data, size_t size);

    private:
        /**
          * The text instance variable is used to remember what to add
          * to the end of each line.
          */
        std::string text;

        /**
          * The column instance variable is used to remember the current
          * output column position.
          */
        size_t column;

        /**
          * The default constructor.  Do not use.
          */
        output_filter_suffix_add();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        output_filter_suffix_add(const output_filter_suffix_add &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        output_filter_suffix_add &operator=(
            const output_filter_suffix_add &rhs);
    };
}

#endif // LIBUNCIA_OUTPUT_FILTER_SUFFIX_ADD_H
