//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <libuncia/output/filter/unexpand.h>


libuncia::output_filter_unexpand::~output_filter_unexpand()
{
}


libuncia::output_filter_unexpand::output_filter_unexpand(
        const pointer &a_deeper, bool a_leading_only, int a_tab_width) :
    output_filter(a_deeper),
    leading_only(a_leading_only),
    tab_width(a_tab_width >= 1 ? a_tab_width : 8),
    out_column(0),
    in_column(0)
{
}


libuncia::output::pointer
libuncia::output_filter_unexpand::create(const pointer &a_deeper,
    bool a_leading_only, int a_tab_width)
{
    return
        pointer
        (
            new output_filter_unexpand(a_deeper, a_leading_only, a_tab_width)
        );
}


ssize_t
libuncia::output_filter_unexpand::write(const void *data, size_t size)
{
    const char *start = (const char *)data;
    const char *end = start + size;
    const char *cp = start;
    while (cp < end)
    {
        char c = *cp++;
        switch (c)
        {
        case ' ':
            ++in_column;
            continue;

        case '\t':
            in_column = ((in_column / tab_width) + 1) * tab_width;
            continue;

        case '\n':
        case '\r':
            in_column = 0;
            break;

        default:
            if (!leading_only || out_column == 0)
            {
                while (out_column < in_column)
                {
                    int ocol2 = ((out_column / tab_width) + 1) * tab_width;
                    if (ocol2 > in_column)
                        break;
                    if (ocol2 - out_column < 2)
                        break;
                    deeper_putc('\t');
                    out_column = ocol2;
                }
            }
            while (out_column < in_column)
            {
                deeper_putc(' ');
                ++out_column;
            }
            ++in_column;
            break;
        }
        deeper_putc(c);
        out_column = in_column;
    }
    return size;
}
