//
// uncia - a big cat
// Copyright (C) 2001-2006, 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <libuncia/output/filter/uuencode.h>


static char etab[] =
    "`!\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_";


libuncia::output_filter_uuencode::~output_filter_uuencode()
{
    if (begun)
    {
        while (residual_bits > 0)
        {
            if (residual_bits < 6)
            {
                // fake a NUL character
                residual_value <<= 8;
                residual_bits += 8;
            }

            residual_bits -= 6;
            int n = (residual_value >> residual_bits);
            n &= 0x3F;
            obuf[opos++] = etab[n];
        }
        if (ipos)
        {
            obuf[0] = etab[ipos];
            obuf[opos++] = '\n';
            deeper_write(obuf, opos);
            ipos = 0;
            opos = 1; // The length character
        }
        deeper_puts("`\nend\n");
    }
}


libuncia::output_filter_uuencode::output_filter_uuencode(
        const pointer &a_deeper) :
    output_filter(a_deeper),
    residual_value(0),
    residual_bits(0),
    opos(1), // The length character
    ipos(0),
    pos(0),
    bol(true),
    begun(false)
{
    //
    // Figure out what to call ourself,
    // discarding any ".uu*" suffix.
    //
    std::string fn = output_filter::filename_get();
    const char *cp1 = fn.c_str();
    const char *cp2 = strrchr(cp1, '/');
    if (cp2)
        ++cp2;
    else
        cp2 = cp1;
    const char *cp3 = strstr(cp2, ".uu");
    if (cp3)
        file_name = std::string(cp1, cp3 - cp1);
    else
        file_name = fn;
}


libuncia::output::pointer
libuncia::output_filter_uuencode::create(const pointer &a_deeper)
{
    return pointer(new output_filter_uuencode(a_deeper));
}


ssize_t
libuncia::output_filter_uuencode::write(const void *p, size_t len)
{
    if (!begun)
    {
        begun = true;
        deeper_puts("begin 644 ");
        deeper_puts(file_name);
        deeper_putc('\n');
    }
    const unsigned char *data = (const unsigned char *)p;
    if (len > 0)
        bol = (data[len - 1] == '\n');
    pos += len;
    while (len > 0)
    {
        unsigned char c = *data++;
        --len;
        residual_value = (residual_value << 8) | c;
        ++ipos;
        residual_bits += 8;
        for (;;)
        {
            residual_bits -= 6;
            int n = (residual_value >> residual_bits);
            n &= 0x3F;
            obuf[opos++] = etab[n];
            if (residual_bits == 0 && opos > 60)
            {
                obuf[0] = etab[ipos];
                obuf[opos++] = '\n';
                deeper_write(obuf, opos);
                opos = 1; // The length character
                ipos = 0;
            }
            if (residual_bits < 6)
                break;
        }
    }
    return len;
}


std::string
libuncia::output_filter_uuencode::filename_get()
    const
{
    return file_name;
}
