//
// uncia - a big cat
// Copyright (C) 2001, 2002, 2005, 2006, 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_OUTPUT_FILTER_UUENCODE_H
#define LIBUNCIA_OUTPUT_FILTER_UUENCODE_H

#include <libuncia/ostream.h>
#include <libuncia/output/filter.h>

namespace libuncia
{
    /**
      * The output_filter_uuencode class is used to represent an output
      * filter which encodes using the Unix-to-Unix encoding.
      */
    class output_filter_uuencode:
        public output_filter
    {
    public:
        /**
          * The destructor.
          */
        virtual ~output_filter_uuencode();

    private:
        /**
          * The constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param deeper
          *     where to send the output of this filter.
          */
        output_filter_uuencode(const pointer &deeper);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper
          *     where to send the output of this filter.
          */
        static pointer create(const pointer &deeper);

    protected:
        // See base class for documentation.
        ssize_t write(const void *data, size_t size);

        // See base class for documentation.
        std::string filename_get() const;

    private:
        /**
          * The residual_value instance variable is used to remember the
          * cumulative bits, accumulated 8 bits per inoput characters,
          * depleted at 6 bits per output character.
          */
        unsigned residual_value;

        /**
          * The residual_bits instance variable is used to remember the
          * number of bits accumulated fo far in the residual_value.
          */
        int residual_bits;

        /**
          * The obuf instance variable is used to remember the output line
          * being accumulated.
          */
        char obuf[64 * 4 / 3];

        /**
          * The opos instance variable is used to remember how much of the
          * obuf has been consumed to date, the numer of output characters.
          */
        int opos;

        /**
          * The ipos instance variable is used to remember how many input
          * characters have been consumed to date in building the obuf.
          */
        int ipos;

        /**
          * The pos instance variable is used to remember the current output
          * file position.
          */
        long pos;

        /**
          * The bol instance variable is used to remember whether or not we
          * are at the start of a line.
          */
        bool bol;

        /**
          * The begun instance variable is used to remember whether or not
          * we have written the "begin" line yet.
          */
        bool begun;

        /**
          * The file_name instance variable is used to remember the name of
          * the input file, with any ".uu*" suffix removed.
          */
        std::string file_name;

        /**
          * The default constructor.  Do not use.
          */
        output_filter_uuencode();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        output_filter_uuencode(const output_filter_uuencode &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        output_filter_uuencode &operator=(const output_filter_uuencode &rhs);
    };
}

#endif // LIBUNCIA_OUTPUT_FILTER_UUENCODE_H
