//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_OUTPUT_NULL_H
#define LIBUNCIA_OUTPUT_NULL_H

#include <libuncia/output.h>

namespace libuncia
{
    /**
      * The output_null class is used to represent the processing
      * necessary to simulate an output which discards all data.
      */
    class output_null:
        public output
    {
    public:
        /**
          * The destructor.
          */
        virtual ~output_null();

    private:
        /**
          * The default constructor.  It is private on purpose, use the
          * #create class method instead.
          */
        output_null();

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          */
        static pointer create();

    protected:
        // See base class for documentation.
        ssize_t write(const void *data, size_t size);

        // See base class for documentation.
        int sync();

        // See base class for documentation.
        std::string filename_get() const;

    private:
        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        output_null(const output_null &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        output_null &operator=(const output_null &rhs);
    };
}

#endif // LIBUNCIA_OUTPUT_NULL_H
