//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_OUTPUT_OSTREAM_H
#define LIBUNCIA_OUTPUT_OSTREAM_H

#include <libuncia/output.h>

namespace libuncia
{
    /**
      * The output_ostream class is used to represent an adapter to
      * allow a std::ostream to be chained into the libuncia::output
      * filters.
      */
    class output_ostream:
        public output
    {
    public:
        /**
          * The destructor.
          */
        virtual ~output_ostream();

    private:
        /**
          * The constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param deeper
          *     where to send the output of this adapter.
          *
          * @note
          *     The referenced std::ostream object <b>must</b> have a
          *     lifetime exceeding that of this object.
          */
        output_ostream(std::ostream &deeper);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper
          *     where to send the output of this adapter.
          *
          * @note
          *     The referenced std::ostream object <b>must</b> have a
          *     lifetime exceeding that of the returned object.
          */
        static pointer create(std::ostream &deeper);

    protected:
        // See base class for documentation.
        ssize_t write(const void *data, size_t size);

        // See base class for documentation.
        int sync();

        // See base class for documentation.
        std::string filename_get() const;

    private:
        /**
          * The deeper instance variable is used to remember where to
          * send the output of this adapter.
          *
          * @note
          *     The referenced std::ostream object <b>must</b> have a
          *     lifetime exceeding that of this object.
          */
        std::ostream &deeper;

        /**
          * The default constructor.  Do not use.
          */
        output_ostream();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        output_ostream(const output_ostream &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        output_ostream &operator=(const output_ostream &rhs);
    };
}

#endif // LIBUNCIA_OUTPUT_OSTREAM_H
