//
//      uncia - a big cat
//      Copyright (C) 2008 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#include <boost/algorithm/string/predicate.hpp>

#include <libuncia/error_handler.h>
#include <libuncia/output/stdout.h>


libuncia::output_stdout::~output_stdout()
{
}


libuncia::output_stdout::output_stdout() :
    filename("standard output")
{
    //
    // On Linx we are able to grope out file descriptors to locate the
    // information withing the /proc file system.  This reflects the
    // kernel's information about our file descriptors.  Try to use that
    // information to locate the filename associated with stdout, so
    // that we can give much better error messages.
    //
    char buf[2000];
    int n = readlink("/dev/stdout", buf, sizeof(buf));
    if (n > 0)
    {
        std::string l1(buf, n);
        if (boost::starts_with(l1, "/proc/self/fd/"))
        {
            n = readlink(l1.c_str(), buf, sizeof(buf));
            if (n > 0)
            {
                std::string l2(buf, n);
                if
                (
                    !boost::starts_with(l2, "/proc/")
                &&
                    !boost::starts_with(l2, "/dev/")
                &&
                    !boost::starts_with(l2, "pipe:")
                )
                    filename = l2;
            }
        }
    }
}


libuncia::output::pointer
libuncia::output_stdout::create()
{
    return pointer(new output_stdout());
}


ssize_t
libuncia::output_stdout::write(const void *data, size_t size)
{
    ssize_t ret = ::write(1, data, size);
    if (ret < 0)
    {
        error_handler_get()->fatal_error_errno("write %s", filename.c_str());
    }
    if (ret != (ssize_t)size)
    {
        error_handler_get()->fatal_error
        (
            "write %s: too short, asked to write %ld, "
                "but only wrote %ld",
            filename.c_str(),
            (long)size,
            (long)ret
        );
    }
    return size;
}


int
libuncia::output_stdout::sync()
{
    return 0;
}


std::string
libuncia::output_stdout::filename_get()
    const
{
    return filename;
}
