//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_OUTPUT_STDOUT_H
#define LIBUNCIA_OUTPUT_STDOUT_H

#include <string>

#include <libuncia/output.h>

namespace libuncia
{
    /**
      * The output_stdout class is used to represent the standard output
      * as a data sink.
      */
    class output_stdout:
        public output
    {
    public:
        /**
          * The destructor.
          */
        virtual ~output_stdout();

    private:
        /**
          * The default constructor.
          */
        output_stdout();

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          */
        static pointer create();

    protected:
        // See base class for documentation.
        ssize_t write(const void *data, size_t size);

        // See base class for documentation.
        int sync();

        // See base class for documentation.
        std::string filename_get() const;

    private:
        /**
          * The filename instance variable is used to remember the name
          * of the standard output, if available.
          */
        std::string filename;

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        output_stdout(const output_stdout &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        output_stdout &operator=(const output_stdout &rhs);
    };
}

#endif // LIBUNCIA_OUTPUT_STDOUT_H
