//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_OUTPUT_TEE_H
#define LIBUNCIA_OUTPUT_TEE_H

#include <libuncia/ostream.h>
#include <libuncia/output.h>

namespace libuncia
{
    /**
      * The output_tee class is used to represent an output filter which
      * is able to send the same output to two deeper outputs.
      */
    class output_tee:
        public output
    {
    public:
        /**
          * The destructor.
          */
        virtual ~output_tee();

    private:
        /**
          * The constructor.
          * It is private on purpose, use the #create class method instead.
          *
          * @param deeper1
          *     The first sub-output.
          * @param deeper2
          *     The second sub-output.
          */
        output_tee(const pointer &deeper1, const pointer &deeper2);

    public:
        /**
          * The create class method is used to create new dynmically
          * allocated instances of this class.
          *
          * @param deeper1
          *     The first sub-output.
          * @param deeper2
          *     The second sub-output.
          */
        static pointer create(const pointer &deeper1, const pointer &deeper2);

    protected:
        // See base class for documentation.
        ssize_t write(const void *data, size_t size);

        // See base class for documentation.
        int sync();

        // See base class for documentation.
        std::string filename_get() const;

    private:
        /**
          * The deeper1 instance variable is used to remember where to
          * send output.
          */
        libuncia::ostream deeper1;

        /**
          * The deeper2 instance variable is used to remember where to
          * send output.
          */
        libuncia::ostream deeper2;

        /**
          * The default constructor.  Do not use.
          */
        output_tee();

        /**
          * The copy constructor.  Do not use.
          *
          * @param rhs
          *     The right hand side of the initialization.
          */
        output_tee(const output_tee &rhs);

        /**
          * The assignment operator.  Do not use.
          *
          * @param rhs
          *     The right hand side of the assignment.
          */
        output_tee &operator=(const output_tee &rhs);
    };
}

#endif // LIBUNCIA_OUTPUT_TEE_H
