//
//      uncia - a big cat
//      Copyright (C) 2008 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#include <cstdio>
#include <string>

#include <libuncia/progname.h>


static std::string progname;


void
libuncia::progname_set(const char *s)
{
    const char *end = s + strlen(s);
    while (end > s && end[-1] == '/')
        --end;
    const char *start = end;
    while (start > s && start[-1] != '/')
        --start;
    progname = std::string(start, end - start);
}


const char *
libuncia::progname_get()
{
    if (progname.empty())
    {
        //
        // On linux, we have a way to figure out the program's name,
        // even if they didn't call progname_set() from main().
        //
        // NOTE: because the error reporting functions call this
        // function, you SHALL NOT do error reporting from here.  If it
        // fails (and it will on non-linux systems) you shall silently
        // ignore the problem.  (If you call error reporting from here,
        // you will get an infinite recursion.)
        //
        FILE *fp = fopen("/proc/self/cmdline", "r");
        if (fp)
        {
            for (;;)
            {
                int c = getc(fp);
                if (c == EOF || c == 0)
                    break;
                if (c == '/')
                    progname.clear();
                else
                    progname.push_back((char)c);
            }
            fclose(fp);
        }
    }
    return (progname.empty() ? "???" : progname.c_str());
}
