//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <cerrno>
#include <cstdio>
#include <cstring>

#include <libuncia/quit.h>


libuncia::quit::~quit()
{
}


libuncia::quit::quit()
{
}


libuncia::quit::quit(const quit &)
{
}


libuncia::quit &
libuncia::quit::operator=(const quit &)
{
    return *this;
}


void
libuncia::quit::fatal_error(const char *fmt, ...)
{
    va_list ap;
    va_start(ap, fmt);
    fatal_error_v(fmt, ap);
    va_end(ap);
}


void
libuncia::quit::fatal_error_v(const char *fmt, va_list ap)
{
    message_v(fmt, ap);
    exit(1);
}


void
libuncia::quit::fatal_error_errno(const char *fmt, ...)
{
    va_list ap;
    va_start(ap, fmt);
    fatal_error_errno_v(fmt, ap);
    va_end(ap);
}


void
libuncia::quit::fatal_error_errno_v(const char *fmt, va_list ap)
{
    int n = errno;
    char buf[1024];
    vsnprintf(buf, sizeof(buf), fmt, ap);
    fatal_error("%s: %s", buf, strerror(n));
}


void
libuncia::quit::warning(const char *fmt, ...)
{
    va_list ap;
    va_start(ap, fmt);
    warning_v(fmt, ap);
    va_end(ap);
}


void
libuncia::quit::warning_v(const char *fmt, va_list ap)
{
    char buf[1024];
    vsnprintf(buf, sizeof(buf), fmt, ap);
    message("warning: %s", buf);
}


void
libuncia::quit::message(const char *fmt, ...)
{
    va_list ap;
    va_start(ap, fmt);
    message_v(fmt, ap);
    va_end(ap);
}
