//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_QUIT_STDERR_AND_EXIT_H
#define LIBUNCIA_QUIT_STDERR_AND_EXIT_H

#include <libuncia/quit.h>

namespace libuncia
{
    /**
      * The quit_stderr_and_exit class is used to represent an error
      * handler which behaves in the "normal" way: print error messages
      * on the standard error, and exit via the system ::exit function.
      */
    class quit_stderr_and_exit:
        public quit
    {
    public:
        /**
          * The destructor.
          */
        virtual ~quit_stderr_and_exit();

        /**
          * The default constructor.
          */
        quit_stderr_and_exit();

    protected:
        // see base class for documentation
        virtual void exit(int);

        // see base class for documentation
        virtual void message_v(const char *, va_list);

    private:
        /**
          * The copy constructor.  Do not use.
          */
        quit_stderr_and_exit(const quit_stderr_and_exit &);

        /**
          * The assignment operator.  Do not use.
          */
        quit_stderr_and_exit &operator=(const quit_stderr_and_exit &);
    };
}

#endif // LIBUNCIA_QUIT_STDERR_AND_EXIT_H
