//
// uncia - a big cat
// Copyright (C) 2000, 2002, 2003, 2006-2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIBUNCIA_QUIT_THROW_EXCEPTION_H
#define LIBUNCIA_QUIT_THROW_EXCEPTION_H

#include <string>

#include <libuncia/quit.h>

namespace libuncia
{
    /**
      * The quit_exception class is use to represent a quit handler which
      * throws a std::runtime_error exception, rather than call ::exit.
      */
    class quit_exception:
        public quit
    {
    public:
        /**
          * The destructor.
          */
        virtual ~quit_exception();

        /**
          * The default constructor.
          */
        quit_exception();

    protected:
        // see base class for documentation
        virtual void exit(int n);

        // see base class for documentation
        virtual void message_v(const char *fmt, va_list);

    private:
        /**
          * The text instance variable is used to remember the text
          * accumulated by this error handler prior to the #exit method
          * being called.  It will be cleared immediately before the
          * exception is thrown, so that multiple exception may be
          * thrown by this one error handler.
          */
        std::string text;

        /**
          * The copy constructor.  Do not use.
          */
        quit_exception(const quit_exception &);

        /**
          * The assignment operator.  Do not use.
          */
        quit_exception &operator=(const quit_exception &);
    };
}

#endif // LIBUNCIA_QUIT_THROW_EXCEPTION_H
