//
//      uncia - a big cat
//      Copyright (C) 2008 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#include <cerrno>
#include <iostream>
#include <libuncia/progname.h>

#include <uncia/argument_source/file.h>


argument_source_file::~argument_source_file()
{
}


argument_source_file::argument_source_file(const std::string &fn) :
    filename(fn),
    deeper(fn.c_str()),
    line_number(1),
    prev_available(false)
{
    if (!deeper)
    {
        int err = errno;
        std::cerr << libuncia::progname_get() << ": open " << filename << ": "
            << strerror(err) << std::endl;
        exit(1);
    }
}


argument_source_file::pointer
argument_source_file::create(const std::string &fn)
{
    return pointer(new argument_source_file(fn));
}



void
argument_source_file::error(const char *text)
    const
{
    std::cerr << libuncia::progname_get() << ": " << filename << ": "
        << line_number << ": " << text;
    if (prev_available)
        std::cerr << " (near " << prev_token << ")";
    std::cerr << std::endl;
}


bool
argument_source_file::get(std::string &token)
{
    prev_available = false;
    prev_token.clear();
    for (;;)
    {
        char c = 0;
        if (!deeper.get(c))
            return false;
        switch (c)
        {
        case '\n':
            ++line_number;
            continue;

        case ' ':
        case '\f':
        case '\r':
        case '\t':
        case '\v':
            //
            // ignore white space
            //
            continue;

        case '#':
            //
            // ignore comments
            //
            for (;;)
            {
                if (!deeper.get(c))
                    break;
                if (c == '\n')
                    break;
            }
            continue;

        default:
            //
            // collect the argument
            //
            token.clear();
            for (;;)
            {
                prev_token.push_back(c);
                if (!deeper.get(c))
                    break;
                switch (c)
                {
                default:
                    continue;

                case ' ':
                case '\f':
                case '\n':
                case '\r':
                case '\t':
                case '\v':
                case '#':
                    deeper.putback(c);
                    break;
                }
                break;
            }
            token = prev_token;
            prev_available = true;
            return true;
        }
    }
}
