//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef UNCIA_ARGUMENT_SOURCE_FILE_H
#define UNCIA_ARGUMENT_SOURCE_FILE_H

#include <fstream>
#include <uncia/argument_source.h>

/**
  * The argument_source_file class is used to represent the processing
  * necessary to pretend a file is actually a source of command line
  * arguments.
  */
class argument_source_file:
    public argument_source
{
public:
    /**
      * The destructor.
      */
    virtual ~argument_source_file();

private:
    /**
      * The constructor.
      * It is private on purpose, use the #create class method instead.
      *
      * @param filename
      *     The name of the file to read and parse.
      */
    argument_source_file(const std::string &filename);

public:
    /**
      * The create class method is used to create new dynmically
      * allocated instances of this class.
      *
      * @param filename
      *     The name of the file to read and parse.
      */
    static pointer create(const std::string &filename);

protected:
    // See base class for documentation.
    bool get(std::string &token);

    // See base class for documentation.
    void error(const char *text) const;

private:
    /**
      * The filename instance variable is used to remember the name of
      * the file being read.
      */
    std::string filename;

    /**
      * The deeper instance variable is used to remember the source of
      * the stream of characters to be parsed.
      */
    std::ifstream deeper;

    /**
      * The line_number instance variable is used to remember the
      * current line in the file.  Used to provide better error
      * messages.
      */
    long line_number;

    /**
      * The prev_token instance variable is used to remember the last
      * token returned by the #get method.
      */
    std::string prev_token;

    /**
      * The prev_available instance variable is used to remember whether
      * or not the #prev_tooken instance variable contains anything
      * useful.
      */
    bool prev_available;

    /**
      * The default constructor.  Do not use.
      */
    argument_source_file();

    /**
      * The copy constructor.  Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    argument_source_file(const argument_source_file &rhs);

    /**
      * The assignment operator.  Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    argument_source_file &operator=(const argument_source_file &rhs);
};

#endif // UNCIA_ARGUMENT_SOURCE_FILE_H
