//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef UNCIA_ARGUMENT_SOURCE_GNU_H
#define UNCIA_ARGUMENT_SOURCE_GNU_H

#include <list>

#include <uncia/argument_source.h>

/**
  * The argument_source_gnu class is used to represent the processing
  * required to massage GNU style long arguments into shorter arguments.
  */
class argument_source_gnu:
    public argument_source
{
public:
    /**
      * The destructor.
      */
    virtual ~argument_source_gnu();

private:
    /**
      * The default constructor.
      * It is private on purpose, use the #create class method instead.
      *
      * @param deeper
      *     The underlying source of command line arguments.
      */
    argument_source_gnu(const pointer &deeper);

public:
    /**
      * The create class method is used to create new dynmically
      * allocated instances of this class.
      *
      * @param deeper
      *     The underlying source of command line arguments.
      */
    static pointer create(const pointer &deeper);

protected:
    // See base class for documentation.
    bool get(std::string &token);

    // See base class for documentation.
    void error(const char *text) const;

private:
    /**
      * The deeper instance variable is used to remember the source of
      * arguments to be filtered.
      */
    pointer deeper;

    /**
      * The holding_pen instance variable is used to remember arguments
      * which have been "pushed back" for return by a later #get call.
      */
    std::list<std::string> holding_pen;

    /**
      * The default constructor.  Do not use.
      */
    argument_source_gnu();

    /**
      * The copy constructor.  Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    argument_source_gnu(const argument_source_gnu &rhs);

    /**
      * The assignment operator.  Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    argument_source_gnu &operator=(const argument_source_gnu &rhs);
};

#endif // UNCIA_ARGUMENT_SOURCE_GNU_H
