//
//      uncia - a big cat
//      Copyright (C) 2008 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#include <uncia/argument_source/file.h>
#include <uncia/argument_source/include.h>


argument_source_include::~argument_source_include()
{
}


argument_source_include::argument_source_include(const pointer &deeper)
{
    src.push_back(deeper);
}


argument_source_include::pointer
argument_source_include::create(const pointer &deeper)
{
    return pointer(new argument_source_include(deeper));
}


bool
argument_source_include::get(std::string &token)
{
    for (;;)
    {
        if (src.empty())
            return false;
        pointer p = src.back();
        if (!p->get(token))
        {
            if (src.size() <= 1)
            {
                // we keep the first one so we can report errors
                return false;
            }
            src.pop_back();
            continue;
        }
        if (token.size() < 2 || token[0] != '@')
            return true;
        std::string fn(token.substr(1, token.size() - 1));
        p = argument_source_file::create(fn);
        src.push_back(p);
    }
}


void
argument_source_include::error(const char *text)
    const
{
    assert(!src.empty());
    src.back()->error(text);
}
