//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef UNCIA_ARGUMENT_SOURCE_INCLUDE_H
#define UNCIA_ARGUMENT_SOURCE_INCLUDE_H

#include <list>
#include <uncia/argument_source.h>

/**
  * The argument_source_include class is used to represent the parse
  * state of the command line include processor.  It intercepts
  * '@filename' arguments and inserts the file's contentas at that
  * point.
  */
class argument_source_include:
    public argument_source
{
public:
    /**
      * The destructor.
      */
    virtual ~argument_source_include();

private:
    /**
      * The constructor.
      * It is private on purpose, use the #create method instead.
      *
      * @param deeper
      *     The underlying source of command line arguments.
      */
    argument_source_include(const pointer &deeper);

public:
    /**
      * The create class method is used to create new dynmically
      * allocated instances of this class.
      *
      * @param deeper
      *     The underlying source of command line arguments.
      */
    static pointer create(const pointer &deeper);

protected:
    // See base class for documentation.
    bool get(std::string &token);

    // See base class for documentation.
    void error(const char *text) const;

private:
    typedef std::list<pointer> src_t;

    /**
      * The src instance variable is used to remember the push-down
      * stack of command line argument sources being processed.
      */
    src_t src;

    /**
      * The default constructor.  Do not use.
      */
    argument_source_include();

    /**
      * The copy constructor.  Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    argument_source_include(const argument_source_include &rhs);

    /**
      * The assignment operator.  Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    argument_source_include &operator=(const argument_source_include &rhs);
};

#endif // UNCIA_ARGUMENT_SOURCE_INCLUDE_H
