//
//      uncia - a big cat
//      Copyright (C) 2008 Peter Miller
//
//      This program is free software; you can redistribute it and/or
//      modify it under the terms of the GNU General Public License,
//      version 3, as published by the Free Software Foundation.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#include <iostream>
#include <libuncia/progname.h>

#include <uncia/argument_source/main.h>


argument_source_main::~argument_source_main()
{
    argc = 0;
    argv = 0;
    position = 0;
}


argument_source_main::argument_source_main() :
    argc(0),
    argv(0),
    position(0)
{
}


argument_source_main::argument_source_main(int ac, char **av) :
    argc(ac),
    argv(av),
    position(1)
{
}


argument_source::pointer
argument_source_main::create(int ac, char **av)
{
    return pointer(new argument_source_main(ac, av));
}


bool
argument_source_main::get(std::string &token)
{
    assert(position <= argc);
    if (position >= argc)
        return false;
    assert(position >= 0);
    token = argv[position++];
    return true;
}


void
argument_source_main::error(const char *text)
    const
{
    std::string s;
    int n = position - 1;
    if (n >= 0 && n < argc)
    {
        unsigned max = 20;
        s = argv[n];
        if (s.empty())
            s = "empty string";
        else if (s.size() > max)
        {
            s = "..." + std::string(s.c_str() + s.size() - max, max);
        }
    }
    if (!s.empty())
        s = " (near " + s + ")";
    std::cerr << libuncia::progname_get() << ": argv[" << n << "]: " << text
        << s << std::endl;
}
