//
// uncia - a big cat
// Copyright (C) 2008 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef UNCIA_ARGUMENT_SOURCE_MAIN_H
#define UNCIA_ARGUMENT_SOURCE_MAIN_H

#include <uncia/argument_source.h>

/**
  * The argument_source_main class is used to represent the command line
  * arguments presented to main() by the operating system.
  */
class argument_source_main:
    public argument_source
{
public:
    /**
      * The destructor.
      */
    virtual ~argument_source_main();

private:
    /**
      * The constructor.  It is private on purpose, use the
      * #create class method instead.
      *
      * @param argc
      *     The number of command line arguments passed to main.
      * @param argv
      *     The command line arguments passed to main.
      */
    argument_source_main(int argc, char **argv);

public:
    /**
      * The create class method is used to create new dynmically
      * allocated instances of this class.
      */
    static pointer create(int argc, char **argv);

protected:
    // See base class for doumentation.
    bool get(std::string &token);

    // See base class for doumentation.
    void error(const char *text) const;

private:
    /**
      * The argc instance variable is used to remember the number of
      * arguments on the command line.
      */
    int argc;

    /**
      * The argv instance variable is used to remember the strings
      * corresponding to the arguments on the command line.
      */
    char **argv;

    /**
      * The position instance variable is used to remember the current
      * "read" point in the list.
      */
    int position;

    /**
      * The default constructor.  Do not use.
      */
    argument_source_main();

    /**
      * The copy constructor.  Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    argument_source_main(const argument_source_main &rhs);

    /**
      * The assignment operator.  Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    argument_source_main &operator=(const argument_source_main &rhs);
};

#endif // UNCIA_ARGUMENT_SOURCE_MAIN_H
