/*
 * uncia - a big cat
 * Copyright (C) 2008 Peter Miller
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License, version 3, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program. If not, see <http://www.gnu.org/licenses/>.
 */

%{

#include <libuncia/input/catenate.h>
#include <libuncia/input/echo.h>
#include <libuncia/input/file.h>
#include <libuncia/input/filter/base64.h>
#include <libuncia/input/filter/column_remove.h>
#include <libuncia/input/filter/crlf.h>
#include <libuncia/input/filter/downcase.h>
#include <libuncia/input/filter/escape_newline.h>
#include <libuncia/input/filter/expand.h>
#include <libuncia/input/filter/gunzip.h>
#include <libuncia/input/filter/hash_comments.h>
#include <libuncia/input/filter/head.h>
#include <libuncia/input/filter/number_lines.h>
#include <libuncia/input/filter/quoted_printable.h>
#include <libuncia/input/filter/remove_prefix.h>
#include <libuncia/input/filter/reverse.h>
#include <libuncia/input/filter/reverse_lines.h>
#include <libuncia/input/filter/rot13.h>
#include <libuncia/input/filter/rot47.h>
#include <libuncia/input/filter/sort.h>
#include <libuncia/input/filter/tail.h>
#include <libuncia/input/filter/upcase.h>
#include <libuncia/input/filter/uudecode.h>
#include <libuncia/input/filter/unique.h>
#include <libuncia/input/paste.h>
#include <libuncia/input/stdin.h>
#include <libuncia/istream.h>
#include <libuncia/ostream.h>
#include <libuncia/output/file.h>
#include <libuncia/output/filter/base64.h>
#include <libuncia/output/filter/crlf.h>
#include <libuncia/output/filter/double_space.h>
#include <libuncia/output/filter/downcase.h>
#include <libuncia/output/filter/fold.h>
#include <libuncia/output/filter/gzip.h>
#include <libuncia/output/filter/hexdump.h>
#include <libuncia/output/filter/number_lines.h>
#include <libuncia/output/filter/prefix.h>
#include <libuncia/output/filter/quoted_printable.h>
#include <libuncia/output/filter/reverse.h>
#include <libuncia/output/filter/rot13.h>
#include <libuncia/output/filter/rot47.h>
#include <libuncia/output/filter/show_nonprinting.h>
#include <libuncia/output/filter/show_tabs.h>
#include <libuncia/output/filter/squeeze_blank.h>
#include <libuncia/output/filter/suffix_add.h>
#include <libuncia/output/filter/unexpand.h>
#include <libuncia/output/filter/uuencode.h>
#include <libuncia/output/filter/upcase.h>
#include <libuncia/output/stdout.h>
#include <libuncia/output/tee.h>
#include <libuncia/print_version.h>

#include <uncia/gram.h>
#include <uncia/lex.h>

#define YYDEBUG 1

%}

%union
{
    libuncia::input::pointer *oipp;
    libuncia::output::pointer *oopp;
    std::string *ssp;
    long lv_integer;
}

%token INTEGER_CONSTANT
%token LP
%token OPTION_BASE64
%token OPTION_COLRM
%token OPTION_CRLF
%token OPTION_DOUBLE_SPACE
%token OPTION_DOWNCASE
%token OPTION_ECHO
%token OPTION_ESCAPE_NEWLINE
%token OPTION_EXPAND
%token OPTION_FOLD
%token OPTION_FOLD_SPACES
%token OPTION_GUNZIP
%token OPTION_GZIP
%token OPTION_HASH_COMMENTS
%token OPTION_HEAD
%token OPTION_HEXDUMP
%token OPTION_NUMBER
%token OPTION_NUMBER_NONBLANK
%token OPTION_OUTPUT
%token OPTION_PASTE
%token OPTION_PASTE_DELIM
%token OPTION_PREFIX_ADD
%token OPTION_PREFIX_REMOVE
%token OPTION_QUOTED_PRINTABLE
%token OPTION_REVERSE_CHARS
%token OPTION_REVERSE_LINES
%token OPTION_ROT13
%token OPTION_ROT47
%token OPTION_SHOW_ALL
%token OPTION_SHOW_ENDS
%token OPTION_SHOW_NONPRINTING
%token OPTION_SHOW_TABS
%token OPTION_SORT
%token OPTION_SQUEEZE_BLANK
%token OPTION_SUFFIX_ADD
%token OPTION_SUFFIX_REMOVE
%token OPTION_TAIL
%token OPTION_UNEXPAND
%token OPTION_UNEXPAND_ALL
%token OPTION_UNIQUE
%token OPTION_UNIQUE_COUNT
%token OPTION_UNKNOWN
%token OPTION_UPCASE
%token OPTION_UUDECODE
%token OPTION_UUENCODE
%token OPTION_YYDEBUG
%token RP
%token STDIO
%token STRING_CONSTANT
%token TEE
%token VERSION

%type <oopp> fold_output
%type <oipp> input
%type <oipp> input_expand
%type <oipp> input_list
%type <oipp> input_optional
%type <lv_integer> INTEGER_CONSTANT
%type <lv_integer> number_or_ten
%type <lv_integer> number
%type <oopp> output
%type <oopp> output_optional
%type <oopp> unexpand_output
%type <ssp> STRING_CONSTANT
%type <oopp> tee

%{


static libuncia::input::pointer *
get_stdin()
{
    static bool stdin_used;
    if (stdin_used)
        yyerror("stdin may only be used once");
    stdin_used = true;
    libuncia::input::pointer result = libuncia::input_stdin::create();
    return new libuncia::input::pointer(result);
}


static libuncia::output::pointer *
get_stdout()
{
    static bool stdout_used;
    if (stdout_used)
        yyerror("stdout may only be used once");
    stdout_used = true;
    libuncia::output::pointer result = libuncia::output_stdout::create();
    return new libuncia::output::pointer(result);
}


void
parse_command_line(int argc, char **argv)
{
    lex_open(argc, argv);
    extern int yyparse();
    yyparse();
    lex_close();
}

%}

%%

uncia
    : VERSION must_be_empty
        {
            libuncia::print_version();
        }
    | settings input_optional output_optional must_be_empty
        {
            libuncia::istream in(*$2);
            delete $2;
            libuncia::ostream out(*$3);
            delete $3;

            //
            // copy the input to the output
            //
            out << in.rdbuf();
        }
    ;

settings
    : /* empty */
    | settings setting
    ;

setting
    : OPTION_YYDEBUG
        { yydebug = 1; }
    ;

input_optional
    : /* empty */
        { $$ = get_stdin(); }
    | input_list
        { $$ = $1; }
    ;

input_list
    : input
        { $$ = $1; }
    | input_list input
        {
            libuncia::input::pointer result =
                libuncia::input_catenate::create(*$1, *$2);
            delete $1;
            delete $2;
            $$ = new libuncia::input::pointer(result);
        }
    ;

input
    : STDIO
        { $$ = get_stdin(); }
    | OPTION_ECHO STRING_CONSTANT
        {
            libuncia::input::pointer result = libuncia::input_echo::create(*$2);
            delete $2;
            $$ = new libuncia::input::pointer(result);
        }
    | STRING_CONSTANT
        {
            libuncia::input::pointer result = libuncia::input_file::create(*$1);
            delete $1;
            $$ = new libuncia::input::pointer(result);
        }
    | LP input RP
        { $$ = $2; }
    | input OPTION_BASE64
        {
            libuncia::input::pointer result =
                libuncia::input_filter_base64::create(*$1);
            delete $1;
            $$ = new libuncia::input::pointer(result);
        }
    | input OPTION_CRLF
        {
            libuncia::input::pointer result =
                libuncia::input_filter_crlf::create(*$1);
            delete $1;
            $$ = new libuncia::input::pointer(result);
        }
    | input OPTION_DOWNCASE
        {
            libuncia::input::pointer result =
                libuncia::input_filter_downcase::create(*$1);
            delete $1;
            $$ = new libuncia::input::pointer(result);
        }
    | input OPTION_ESCAPE_NEWLINE
        {
            libuncia::input::pointer result =
                libuncia::input_filter_escape_newline::create(*$1);
            delete $1;
            $$ = new libuncia::input::pointer(result);
        }
    | input_expand
        { $$ = $1; }
    | input OPTION_GUNZIP
        {
            libuncia::input::pointer result =
                libuncia::input_filter_gunzip::create(*$1);
            delete $1;
            $$ = new libuncia::input::pointer(result);
        }
    | input OPTION_HASH_COMMENTS
        {
            libuncia::input::pointer result =
                libuncia::input_filter_hash_comments::create(*$1);
            delete $1;
            $$ = new libuncia::input::pointer(result);
        }
    | input OPTION_HEAD number_or_ten
        {
            libuncia::input::pointer result =
                libuncia::input_filter_head::create(*$1, $3);
            delete $1;
            $$ = new libuncia::input::pointer(result);
        }
    | input OPTION_NUMBER
        {
            libuncia::input::pointer result =
                libuncia::input_filter_number_lines::create(*$1, true);
            delete $1;
            $$ = new libuncia::input::pointer(result);
        }
    | input OPTION_NUMBER_NONBLANK
        {
            libuncia::input::pointer result =
                libuncia::input_filter_number_lines::create(*$1, false);
            delete $1;
            $$ = new libuncia::input::pointer(result);
        }
    | input OPTION_PREFIX_REMOVE STRING_CONSTANT
        {
            libuncia::input::pointer result =
                libuncia::input_filter_remove_prefix::create(*$1, *$3);
            delete $1;
            delete $3;
            $$ = new libuncia::input::pointer(result);
        }
    | input OPTION_QUOTED_PRINTABLE
        {
            libuncia::input::pointer result =
                libuncia::input_filter_quoted_printable::create(*$1);
            delete $1;
            $$ = new libuncia::input::pointer(result);
        }
    | input OPTION_REVERSE_CHARS
        {
            libuncia::input::pointer result =
                libuncia::input_filter_reverse::create(*$1);
            delete $1;
            $$ = new libuncia::input::pointer(result);
        }
    | input OPTION_REVERSE_LINES
        {
            libuncia::input::pointer result =
                libuncia::input_filter_reverse_lines::create(*$1);
            delete $1;
            $$ = new libuncia::input::pointer(result);
        }
    | input OPTION_ROT13
        {
            libuncia::input::pointer result =
                libuncia::input_filter_rot13::create(*$1);
            delete $1;
            $$ = new libuncia::input::pointer(result);
        }
    | input OPTION_ROT47
        {
            libuncia::input::pointer result =
                libuncia::input_filter_rot47::create(*$1);
            delete $1;
            $$ = new libuncia::input::pointer(result);
        }
    | input OPTION_TAIL number_or_ten
        {
            libuncia::input::pointer result =
                libuncia::input_filter_tail::create(*$1, $3);
            delete $1;
            $$ = new libuncia::input::pointer(result);
        }
    | input OPTION_UPCASE
        {
            libuncia::input::pointer result =
                libuncia::input_filter_upcase::create(*$1);
            delete $1;
            $$ = new libuncia::input::pointer(result);
        }
    | input OPTION_UNKNOWN
        {
            yyerror("unknown option");
            YYABORT;
        }
    ;

input_expand
    : input OPTION_EXPAND
        {
            libuncia::input::pointer result =
                libuncia::input_filter_expand::create(*$1);
            delete $1;
            $$ = new libuncia::input::pointer(result);
        }
    | input OPTION_EXPAND number
        {
            libuncia::input::pointer result =
                libuncia::input_filter_expand::create(*$1, $3);
            delete $1;
            $$ = new libuncia::input::pointer(result);
        }
    ;

output_optional
    : /* empty */
        { $$ = get_stdout(); }
    | OPTION_OUTPUT output
        { $$ = $2; }
    ;

output
    : STDIO
        { $$ = get_stdout(); }
    | STRING_CONSTANT
        {
            libuncia::output::pointer result =
                libuncia::output_file::create(*$1);
            delete $1;
            $$ = new libuncia::output::pointer(result);
        }
    | LP output RP
        { $$ = $2; }
    | OPTION_BASE64 output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_base64::create(*$2);
            delete $2;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_CRLF output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_crlf::create(*$2);
            delete $2;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_DOWNCASE output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_downcase::create(*$2);
            delete $2;
            $$ = new libuncia::output::pointer(result);
        }
    | fold_output
        { $$ = $1; }
    | OPTION_HEXDUMP output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_hexdump::create(*$2);
            delete $2;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_NUMBER output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_number_lines::create(*$2, true);
            delete $2;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_NUMBER_NONBLANK output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_number_lines::create(*$2, false);
            delete $2;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_PREFIX_ADD STRING_CONSTANT output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_prefix::create(*$3, *$2);
            delete $2;
            delete $3;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_QUOTED_PRINTABLE output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_quoted_printable::create(*$2);
            delete $2;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_REVERSE_CHARS output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_reverse::create(*$2);
            delete $2;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_ROT13 output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_rot13::create(*$2);
            delete $2;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_ROT47 output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_rot47::create(*$2);
            delete $2;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_SHOW_ALL output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_suffix_add::create
                (
                    libuncia::output_filter_show_nonprinting::create
                    (
                        libuncia::output_filter_show_tabs::create(*$2)
                    ),
                    "$"
                );
            delete $2;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_SHOW_ENDS output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_suffix_add::create(*$2, "$");
            delete $2;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_SHOW_NONPRINTING output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_show_nonprinting::create(*$2);
            delete $2;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_SHOW_TABS output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_show_tabs::create(*$2);
            delete $2;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_SUFFIX_ADD STRING_CONSTANT output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_suffix_add::create(*$3, *$2);
            delete $2;
            delete $3;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_SQUEEZE_BLANK output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_squeeze_blank::create(*$2);
            delete $2;
            $$ = new libuncia::output::pointer(result);
        }
    | unexpand_output
        { $$ = $1; }
    | OPTION_UPCASE output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_upcase::create(*$2);
            delete $2;
            $$ = new libuncia::output::pointer(result);
        }
    | tee
        { $$ = $1; }
    | OPTION_UNKNOWN
        {
            yyerror("unknown option");
            YYABORT;
        }
    ;

tee
    : TEE output output
        {
            libuncia::output::pointer result =
                libuncia::output_tee::create(*$2, *$3);
            delete $2;
            delete $3;
            $$ = new libuncia::output::pointer(result);
        }
    ;

must_be_empty
    : /* empty */
    | error
        { YYABORT; }
    ;

number_or_ten
    : INTEGER_CONSTANT
        { $$ = $1; }
    | /* empty */
        { $$ = 10; }
    ;

number
    : INTEGER_CONSTANT
        { $$ = $1; }
    ;

fold_output
    : OPTION_FOLD output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_fold::create(*$2, false, 75, 8);
            delete $2;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_FOLD_SPACES output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_fold::create(*$2, true, 75, 8);
            delete $2;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_FOLD number output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_fold::create(*$3, false, $2, 8);
            delete $3;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_FOLD_SPACES number output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_fold::create(*$3, true, $2, 8);
            delete $3;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_FOLD number number output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_fold::create(*$4, false, $2, $3);
            delete $4;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_FOLD_SPACES number number output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_fold::create(*$4, true, $2, $3);
            delete $4;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_FOLD number number number output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_fold::create(*$5, false, $2, $3, $4);
            delete $5;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_FOLD_SPACES number number number output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_fold::create(*$5, true, $2, $3, $4);
            delete $5;
            $$ = new libuncia::output::pointer(result);
        }
    ;

unexpand_output
    : OPTION_UNEXPAND output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_unexpand::create(*$2, true);
            delete $2;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_UNEXPAND_ALL output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_unexpand::create(*$2, false);
            delete $2;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_UNEXPAND number output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_unexpand::create(*$3, true, $2);
            delete $3;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_UNEXPAND_ALL number output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_unexpand::create(*$3, false, $2);
            delete $3;
            $$ = new libuncia::output::pointer(result);
        }
    ;

output
    : OPTION_GZIP output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_gzip::create(*$2);
            delete $2;
            $$ = new libuncia::output::pointer(result);
        }
    | OPTION_GZIP number output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_gzip::create(*$3, $2);
            delete $3;
            $$ = new libuncia::output::pointer(result);
        }
    ;

input
    : OPTION_PASTE input input
        {
            libuncia::input::pointer result =
                libuncia::input_paste::create(*$2, *$3);
            delete $2;
            delete $3;
            $$ = new libuncia::input::pointer(result);
        }
    | OPTION_PASTE_DELIM STRING_CONSTANT input input
        {
            libuncia::input::pointer result =
                libuncia::input_paste::create(*$2, *$3, *$4);
            delete $2;
            delete $3;
            delete $4;
            $$ = new libuncia::input::pointer(result);
        }
    ;

input
    : input OPTION_UUDECODE
        {
            libuncia::input::pointer result =
                libuncia::input_filter_uudecode::create(*$1);
            delete $1;
            $$ = new libuncia::input::pointer(result);
        }
    ;

output
    : OPTION_DOUBLE_SPACE output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_double_space::create(*$2);
            delete $2;
            $$ = new libuncia::output::pointer(result);
        }
    ;

output
    : OPTION_UUENCODE output
        {
            libuncia::output::pointer result =
                libuncia::output_filter_uuencode::create(*$2);
            delete $2;
            $$ = new libuncia::output::pointer(result);
        }
    ;

input
    : input OPTION_COLRM number
        {
            int left = $3 - 1;
            libuncia::input::pointer result =
                libuncia::input_filter_column_remove::create(*$1, left);
            delete $1;
            $$ = new libuncia::input::pointer(result);
        }
    | input OPTION_COLRM number number
        {
            int left = $3 - 1;
            int right = $4 - 1;
            libuncia::input::pointer result =
                libuncia::input_filter_column_remove::create(*$1, left, right);
            delete $1;
            $$ = new libuncia::input::pointer(result);
        }
    ;

input
    : input OPTION_UNIQUE
        {
            libuncia::input::pointer result =
                libuncia::input_filter_unique::create(*$1, false);
            delete $1;
            $$ = new libuncia::input::pointer(result);
        }
    | input OPTION_UNIQUE_COUNT
        {
            libuncia::input::pointer result =
                libuncia::input_filter_unique::create(*$1, true);
            delete $1;
            $$ = new libuncia::input::pointer(result);
        }
    ;

input
    : input OPTION_SORT
        {
            libuncia::input::pointer result =
                libuncia::input_filter_sort::create(*$1);
            delete $1;
            $$ = new libuncia::input::pointer(result);
        }
    ;
